(*
   Elysion Frameworks
   Branch: Legacy
   Version: 09-03a with some 09-04a improvements

   Copyright (c) 2009 Freeze.Dev (Johannes Stein and contributors)

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation
  files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use,
  copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  OTHER DEALINGS IN THE SOFTWARE.

  Thanks to:
  Sam Latinga - The father of SDL
  Dominique Louis - Who took the time and translated SDL to Pascal
  Robrt Kisnmeth - Thanks a lot for the Sprite Engine. Helped me a lot
                     by setting up TelSpriteEngine and including Z sorting

  http://www.freeze-dev.de
  info@freeze-dev.de

  Please, just use it if you are extremely sure about it. Actually, I created
  this for myself. There could be a lot of API changes in the future.

  *)

unit ElysionLegacy;

{$IFDEF FPC}
	{$MODE DELPHI}
{$ENDIF}

interface

uses SDL,
     SDL_Image,
     SFont,
     SDLUtils,
     SysUtils,
     Classes;

const

// ASCII Keymap
// copied from SDL.pas
  K_UNKNOWN = 0;
{$EXTERNALSYM K_UNKNOWN}
  K_FIRST = 0;
{$EXTERNALSYM K_FIRST}
  K_BACKSPACE = 8;
{$EXTERNALSYM K_BACKSPACE}
  K_TAB = 9;
{$EXTERNALSYM K_TAB}
  K_CLEAR = 12;
{$EXTERNALSYM K_CLEAR}
  K_RETURN = 13;
{$EXTERNALSYM K_RETURN}
  K_PAUSE = 19;
{$EXTERNALSYM K_PAUSE}
  K_ESCAPE = 27;
{$EXTERNALSYM K_ESCAPE}
  K_SPACE = 32;
{$EXTERNALSYM K_SPACE}
  K_EXCLAIM = 33;
{$EXTERNALSYM K_EXCLAIM}
  K_QUOTEDBL = 34;
{$EXTERNALSYM K_QUOTEDBL}
  K_HASH = 35;
{$EXTERNALSYM K_HASH}
  K_DOLLAR = 36;
{$EXTERNALSYM K_DOLLAR}
  K_AMPERSAND = 38;
{$EXTERNALSYM K_AMPERSAND}
  K_QUOTE = 39;
{$EXTERNALSYM K_QUOTE}
  K_LEFTPAREN = 40;
{$EXTERNALSYM K_LEFTPAREN}
  K_RIGHTPAREN = 41;
{$EXTERNALSYM K_RIGHTPAREN}
  K_ASTERISK = 42;
{$EXTERNALSYM K_ASTERISK}
  K_PLUS = 43;
{$EXTERNALSYM K_PLUS}
  K_COMMA = 44;
{$EXTERNALSYM K_COMMA}
  K_MINUS = 45;
{$EXTERNALSYM K_MINUS}
  K_PERIOD = 46;
{$EXTERNALSYM K_PERIOD}
  K_SLASH = 47;
{$EXTERNALSYM K_SLASH}
  K_0 = 48;
{$EXTERNALSYM K_0}
  K_1 = 49;
{$EXTERNALSYM K_1}
  K_2 = 50;
{$EXTERNALSYM K_2}
  K_3 = 51;
{$EXTERNALSYM K_3}
  K_4 = 52;
{$EXTERNALSYM K_4}
  K_5 = 53;
{$EXTERNALSYM K_5}
  K_6 = 54;
{$EXTERNALSYM K_6}
  K_7 = 55;
{$EXTERNALSYM K_7}
  K_8 = 56;
{$EXTERNALSYM K_8}
  K_9 = 57;
{$EXTERNALSYM K_9}
  K_COLON = 58;
{$EXTERNALSYM K_COLON}
  K_SEMICOLON = 59;
{$EXTERNALSYM K_SEMICOLON}
  K_LESS = 60;
{$EXTERNALSYM K_LESS}
  K_EQUALS = 61;
{$EXTERNALSYM K_EQUALS}
  K_GREATER = 62;
{$EXTERNALSYM K_GREATER}
  K_QUESTION = 63;
{$EXTERNALSYM K_QUESTION}
  K_AT = 64;
{$EXTERNALSYM K_AT}

  { Skip uppercase letters }

  K_LEFTBRACKET = 91;
{$EXTERNALSYM K_LEFTBRACKET}
  K_BACKSLASH = 92;
{$EXTERNALSYM K_BACKSLASH}
  K_RIGHTBRACKET = 93;
{$EXTERNALSYM K_RIGHTBRACKET}
  K_CARET = 94;
{$EXTERNALSYM K_CARET}
  K_UNDERSCORE = 95;
{$EXTERNALSYM K_UNDERSCORE}
  K_BACKQUOTE = 96;
{$EXTERNALSYM K_BACKQUOTE}
  K_a = 97;
{$EXTERNALSYM K_a}
  K_b = 98;
{$EXTERNALSYM K_b}
  K_c = 99;
{$EXTERNALSYM K_c}
  K_d = 100;
{$EXTERNALSYM K_d}
  K_e = 101;
{$EXTERNALSYM K_e}
  K_f = 102;
{$EXTERNALSYM K_f}
  K_g = 103;
{$EXTERNALSYM K_g}
  K_h = 104;
{$EXTERNALSYM K_h}
  K_i = 105;
{$EXTERNALSYM K_i}
  K_j = 106;
{$EXTERNALSYM K_j}
  K_k = 107;
{$EXTERNALSYM K_k}
  K_l = 108;
{$EXTERNALSYM K_l}
  K_m = 109;
{$EXTERNALSYM K_m}
  K_n = 110;
{$EXTERNALSYM K_n}
  K_o = 111;
{$EXTERNALSYM K_o}
  K_p = 112;
{$EXTERNALSYM K_p}
  K_q = 113;
{$EXTERNALSYM K_q}
  K_r = 114;
{$EXTERNALSYM K_r}
  K_s = 115;
{$EXTERNALSYM K_s}
  K_t = 116;
{$EXTERNALSYM K_t}
  K_u = 117;
{$EXTERNALSYM K_u}
  K_v = 118;
{$EXTERNALSYM K_v}
  K_w = 119;
{$EXTERNALSYM K_w}
  K_x = 120;
{$EXTERNALSYM K_x}
  K_y = 121;
{$EXTERNALSYM K_y}
  K_z = 122;
{$EXTERNALSYM K_z}
  K_DELETE = 127;
{$EXTERNALSYM K_DELETE}
  // End of ASCII mapped keysyms

  // International keyboard syms
  K_WORLD_0 = 160; // 0xA0
{$EXTERNALSYM K_WORLD_0}
  K_WORLD_1 = 161;
{$EXTERNALSYM K_WORLD_1}
  K_WORLD_2 = 162;
{$EXTERNALSYM K_WORLD_2}
  K_WORLD_3 = 163;
{$EXTERNALSYM K_WORLD_3}
  K_WORLD_4 = 164;
{$EXTERNALSYM K_WORLD_4}
  K_WORLD_5 = 165;
{$EXTERNALSYM K_WORLD_5}
  K_WORLD_6 = 166;
{$EXTERNALSYM K_WORLD_6}
  K_WORLD_7 = 167;
{$EXTERNALSYM K_WORLD_7}
  K_WORLD_8 = 168;
{$EXTERNALSYM K_WORLD_8}
  K_WORLD_9 = 169;
{$EXTERNALSYM K_WORLD_9}
  K_WORLD_10 = 170;
{$EXTERNALSYM K_WORLD_10}
  K_WORLD_11 = 171;
{$EXTERNALSYM K_WORLD_11}
  K_WORLD_12 = 172;
{$EXTERNALSYM K_WORLD_12}
  K_WORLD_13 = 173;
{$EXTERNALSYM K_WORLD_13}
  K_WORLD_14 = 174;
{$EXTERNALSYM K_WORLD_14}
  K_WORLD_15 = 175;
{$EXTERNALSYM K_WORLD_15}
  K_WORLD_16 = 176;
{$EXTERNALSYM K_WORLD_16}
  K_WORLD_17 = 177;
{$EXTERNALSYM K_WORLD_17}
  K_WORLD_18 = 178;
{$EXTERNALSYM K_WORLD_18}
  K_WORLD_19 = 179;
{$EXTERNALSYM K_WORLD_19}
  K_WORLD_20 = 180;
{$EXTERNALSYM K_WORLD_20}
  K_WORLD_21 = 181;
{$EXTERNALSYM K_WORLD_21}
  K_WORLD_22 = 182;
{$EXTERNALSYM K_WORLD_22}
  K_WORLD_23 = 183;
{$EXTERNALSYM K_WORLD_23}
  K_WORLD_24 = 184;
{$EXTERNALSYM K_WORLD_24}
  K_WORLD_25 = 185;
{$EXTERNALSYM K_WORLD_25}
  K_WORLD_26 = 186;
{$EXTERNALSYM K_WORLD_26}
  K_WORLD_27 = 187;
{$EXTERNALSYM K_WORLD_27}
  K_WORLD_28 = 188;
{$EXTERNALSYM K_WORLD_28}
  K_WORLD_29 = 189;
{$EXTERNALSYM K_WORLD_29}
  K_WORLD_30 = 190;
{$EXTERNALSYM K_WORLD_30}
  K_WORLD_31 = 191;
{$EXTERNALSYM K_WORLD_31}
  K_WORLD_32 = 192;
{$EXTERNALSYM K_WORLD_32}
  K_WORLD_33 = 193;
{$EXTERNALSYM K_WORLD_33}
  K_WORLD_34 = 194;
{$EXTERNALSYM K_WORLD_34}
  K_WORLD_35 = 195;
{$EXTERNALSYM K_WORLD_35}
  K_WORLD_36 = 196;
{$EXTERNALSYM K_WORLD_36}
  K_WORLD_37 = 197;
{$EXTERNALSYM K_WORLD_37}
  K_WORLD_38 = 198;
{$EXTERNALSYM K_WORLD_38}
  K_WORLD_39 = 199;
{$EXTERNALSYM K_WORLD_39}
  K_WORLD_40 = 200;
{$EXTERNALSYM K_WORLD_40}
  K_WORLD_41 = 201;
{$EXTERNALSYM K_WORLD_41}
  K_WORLD_42 = 202;
{$EXTERNALSYM K_WORLD_42}
  K_WORLD_43 = 203;
{$EXTERNALSYM K_WORLD_43}
  K_WORLD_44 = 204;
{$EXTERNALSYM K_WORLD_44}
  K_WORLD_45 = 205;
{$EXTERNALSYM K_WORLD_45}
  K_WORLD_46 = 206;
{$EXTERNALSYM K_WORLD_46}
  K_WORLD_47 = 207;
{$EXTERNALSYM K_WORLD_47}
  K_WORLD_48 = 208;
{$EXTERNALSYM K_WORLD_48}
  K_WORLD_49 = 209;
{$EXTERNALSYM K_WORLD_49}
  K_WORLD_50 = 210;
{$EXTERNALSYM K_WORLD_50}
  K_WORLD_51 = 211;
{$EXTERNALSYM K_WORLD_51}
  K_WORLD_52 = 212;
{$EXTERNALSYM K_WORLD_52}
  K_WORLD_53 = 213;
{$EXTERNALSYM K_WORLD_53}
  K_WORLD_54 = 214;
{$EXTERNALSYM K_WORLD_54}
  K_WORLD_55 = 215;
{$EXTERNALSYM K_WORLD_55}
  K_WORLD_56 = 216;
{$EXTERNALSYM K_WORLD_56}
  K_WORLD_57 = 217;
{$EXTERNALSYM K_WORLD_57}
  K_WORLD_58 = 218;
{$EXTERNALSYM K_WORLD_58}
  K_WORLD_59 = 219;
{$EXTERNALSYM K_WORLD_59}
  K_WORLD_60 = 220;
{$EXTERNALSYM K_WORLD_60}
  K_WORLD_61 = 221;
{$EXTERNALSYM K_WORLD_61}
  K_WORLD_62 = 222;
{$EXTERNALSYM K_WORLD_62}
  K_WORLD_63 = 223;
{$EXTERNALSYM K_WORLD_63}
  K_WORLD_64 = 224;
{$EXTERNALSYM K_WORLD_64}
  K_WORLD_65 = 225;
{$EXTERNALSYM K_WORLD_65}
  K_WORLD_66 = 226;
{$EXTERNALSYM K_WORLD_66}
  K_WORLD_67 = 227;
{$EXTERNALSYM K_WORLD_67}
  K_WORLD_68 = 228;
{$EXTERNALSYM K_WORLD_68}
  K_WORLD_69 = 229;
{$EXTERNALSYM K_WORLD_69}
  K_WORLD_70 = 230;
{$EXTERNALSYM K_WORLD_70}
  K_WORLD_71 = 231;
{$EXTERNALSYM K_WORLD_71}
  K_WORLD_72 = 232;
{$EXTERNALSYM K_WORLD_72}
  K_WORLD_73 = 233;
{$EXTERNALSYM K_WORLD_73}
  K_WORLD_74 = 234;
{$EXTERNALSYM K_WORLD_74}
  K_WORLD_75 = 235;
{$EXTERNALSYM K_WORLD_75}
  K_WORLD_76 = 236;
{$EXTERNALSYM K_WORLD_76}
  K_WORLD_77 = 237;
{$EXTERNALSYM K_WORLD_77}
  K_WORLD_78 = 238;
{$EXTERNALSYM K_WORLD_78}
  K_WORLD_79 = 239;
{$EXTERNALSYM K_WORLD_79}
  K_WORLD_80 = 240;
{$EXTERNALSYM K_WORLD_80}
  K_WORLD_81 = 241;
{$EXTERNALSYM K_WORLD_81}
  K_WORLD_82 = 242;
{$EXTERNALSYM K_WORLD_82}
  K_WORLD_83 = 243;
{$EXTERNALSYM K_WORLD_83}
  K_WORLD_84 = 244;
{$EXTERNALSYM K_WORLD_84}
  K_WORLD_85 = 245;
{$EXTERNALSYM K_WORLD_85}
  K_WORLD_86 = 246;
{$EXTERNALSYM K_WORLD_86}
  K_WORLD_87 = 247;
{$EXTERNALSYM K_WORLD_87}
  K_WORLD_88 = 248;
{$EXTERNALSYM K_WORLD_88}
  K_WORLD_89 = 249;
{$EXTERNALSYM K_WORLD_89}
  K_WORLD_90 = 250;
{$EXTERNALSYM K_WORLD_90}
  K_WORLD_91 = 251;
{$EXTERNALSYM K_WORLD_91}
  K_WORLD_92 = 252;
{$EXTERNALSYM K_WORLD_92}
  K_WORLD_93 = 253;
{$EXTERNALSYM K_WORLD_93}
  K_WORLD_94 = 254;
{$EXTERNALSYM K_WORLD_94}
  K_WORLD_95 = 255; // 0xFF
{$EXTERNALSYM K_WORLD_95}

  // Numeric keypad
  K_KP0 = 256;
{$EXTERNALSYM K_KP0}
  K_KP1 = 257;
{$EXTERNALSYM K_KP1}
  K_KP2 = 258;
{$EXTERNALSYM K_KP2}
  K_KP3 = 259;
{$EXTERNALSYM K_KP3}
  K_KP4 = 260;
{$EXTERNALSYM K_KP4}
  K_KP5 = 261;
{$EXTERNALSYM K_KP5}
  K_KP6 = 262;
{$EXTERNALSYM K_KP6}
  K_KP7 = 263;
{$EXTERNALSYM K_KP7}
  K_KP8 = 264;
{$EXTERNALSYM K_KP8}
  K_KP9 = 265;
{$EXTERNALSYM K_KP9}
  K_KP_PERIOD = 266;
{$EXTERNALSYM K_KP_PERIOD}
  K_KP_DIVIDE = 267;
{$EXTERNALSYM K_KP_DIVIDE}
  K_KP_MULTIPLY = 268;
{$EXTERNALSYM K_KP_MULTIPLY}
  K_KP_MINUS = 269;
{$EXTERNALSYM K_KP_MINUS}
  K_KP_PLUS = 270;
{$EXTERNALSYM K_KP_PLUS}
  K_KP_ENTER = 271;
{$EXTERNALSYM K_KP_ENTER}
  K_KP_EQUALS = 272;
{$EXTERNALSYM K_KP_EQUALS}

  // Arrows + Home/End pad
  K_UP = 273;
{$EXTERNALSYM K_UP}
  K_DOWN = 274;
{$EXTERNALSYM K_DOWN}
  K_RIGHT = 275;
{$EXTERNALSYM K_RIGHT}
  K_LEFT = 276;
{$EXTERNALSYM K_LEFT}
  K_INSERT = 277;
{$EXTERNALSYM K_INSERT}
  K_HOME = 278;
{$EXTERNALSYM K_HOME}
  K_END = 279;
{$EXTERNALSYM K_END}
  K_PAGEUP = 280;
{$EXTERNALSYM K_PAGEUP}
  K_PAGEDOWN = 281;
{$EXTERNALSYM K_PAGEDOWN}

  // Function keys
  K_F1 = 282;
{$EXTERNALSYM K_F1}
  K_F2 = 283;
{$EXTERNALSYM K_F2}
  K_F3 = 284;
{$EXTERNALSYM K_F3}
  K_F4 = 285;
{$EXTERNALSYM K_F4}
  K_F5 = 286;
{$EXTERNALSYM K_F5}
  K_F6 = 287;
{$EXTERNALSYM K_F6}
  K_F7 = 288;
{$EXTERNALSYM K_F7}
  K_F8 = 289;
{$EXTERNALSYM K_F8}
  K_F9 = 290;
{$EXTERNALSYM K_F9}
  K_F10 = 291;
{$EXTERNALSYM K_F10}
  K_F11 = 292;
{$EXTERNALSYM K_F11}
  K_F12 = 293;
{$EXTERNALSYM K_F12}
  K_F13 = 294;
{$EXTERNALSYM K_F13}
  K_F14 = 295;
{$EXTERNALSYM K_F14}
  K_F15 = 296;
{$EXTERNALSYM K_F15}

  // Key state modifier keys
  K_NUMLOCK = 300;
{$EXTERNALSYM K_NUMLOCK}
  K_CAPSLOCK = 301;
{$EXTERNALSYM K_CAPSLOCK}
  K_SCROLLOCK = 302;
{$EXTERNALSYM K_SCROLLOCK}
  K_RSHIFT = 303;
{$EXTERNALSYM K_RSHIFT}
  K_LSHIFT = 304;
{$EXTERNALSYM K_LSHIFT}
  K_RCTRL = 305;
{$EXTERNALSYM K_RCTRL}
  K_LCTRL = 306;
{$EXTERNALSYM K_LCTRL}
  K_RALT = 307;
{$EXTERNALSYM K_RALT}
  K_LALT = 308;
{$EXTERNALSYM K_LALT}
  K_RMETA = 309;
{$EXTERNALSYM K_RMETA}
  K_LMETA = 310;
{$EXTERNALSYM K_LMETA}
  K_LSUPER = 311; // Left "Windows" key
{$EXTERNALSYM K_LSUPER}
  K_RSUPER = 312; // Right "Windows" key
{$EXTERNALSYM K_RSUPER}
  K_MODE = 313; // "Alt Gr" key
{$EXTERNALSYM K_MODE}
  K_COMPOSE = 314; // Multi-key compose key
{$EXTERNALSYM K_COMPOSE}

  // Miscellaneous function keys
  K_HELP = 315;
{$EXTERNALSYM K_HELP}
  K_PRINT = 316;
{$EXTERNALSYM K_PRINT}
  K_SYSREQ = 317;
{$EXTERNALSYM K_SYSREQ}
  K_BREAK = 318;
{$EXTERNALSYM K_BREAK}
  K_MENU = 319;
{$EXTERNALSYM K_MENU}
  K_POWER = 320; // Power Macintosh power key
{$EXTERNALSYM K_POWER}
  K_EURO = 321; // Some european keyboards
{$EXTERNALSYM K_EURO}
// End of keys

// And here we go with the mouse buttons
  BUTTON_LEFT      = 1;
{$EXTERNALSYM BUTTON_LEFT}
  BUTTON_MIDDLE    = 2;
{$EXTERNALSYM BUTTON_MIDDLE}
  BUTTON_RIGHT     = 3;
{$EXTERNALSYM BUTTON_RIGHT}
  BUTTON_WHEELUP   = 4;
{$EXTERNALSYM BUTTON_WHEELUP}
  BUTTON_WHEELDOWN = 5;
{$EXTERNALSYM BUTTON_WHEELDOWN}
// End of copying from other units

type

TCoord = record
  X, Y: Single;
end;

TCoordInt = record
  X, Y: Integer;
end;

TRGBColor = Record
    R, G, B: Byte;
end;

TRGBAColor = Record
  R, G, B, Alpha: Byte;
end;

TelRect = Record
  X, Y, W, H: Integer;
end;

TelSurface = class
  private
	  FHeight, FWidth, FColorBits: Integer;
	  FFullscreen: Boolean;
	  FHideCursor: Boolean;


	  FRun: Boolean;
    FFlags, FGFlags: Cardinal;
    FFullscreenFlag: Cardinal;

    UseBackgroundImage: Boolean;
    FBackgroundColor: TRGBColor;
    FBackgroundImage: PSDL_Surface;

    FEvent: TSDL_Event;
    
    FCaption, FIcon: PChar;

    FStartTick, FCurTick, FTickNow: UInt32;
    FFrames, FFrameCount: UInt32;

    FFPSSet: Boolean;
    st, en, diff: UInt32;

    fnext_time: UInt32;
    ffps: Integer;
    f__fps: Integer;
    fnow: UInt32;

    procedure SetDelayTime;
    procedure HandleEvents;
  public
	  SDL_Surface: PSDL_Surface;

	  KeyDown: array[0..K_EURO] of Boolean;
    KeyUp: array[0..K_EURO] of Boolean;
	  Cursor: TCoordInt;
	  MouseButtonDown: array[BUTTON_LEFT..BUTTON_WHEELDOWN] of Boolean;
    MouseButtonUp: array[BUTTON_LEFT..BUTTON_WHEELDOWN] of Boolean;

    constructor Create;
	  function Initialize(Width, Height, ColorBits: Integer; Fullscreen: Boolean): Boolean;
	  procedure SetVideoMode(Width, Height, ColorBits: Integer; Fullscreen: Boolean = False);

	  procedure Finalize;

	  procedure BeginScene;
    procedure EndScene; Overload;

    // Expert function (recommended): Use rects to update surface; Improves performance
    procedure EndScene(UpdateRects: array of TelRect); Overload;

    procedure Quit;

    // Expert function: Use background image instead of a color
    procedure SetBackgroundImage(Surface: PSDL_Surface);
    procedure SetBackgroundColor(Color: TRGBColor);

    procedure SetCaption(Caption: String);
    procedure SetIcon(Icon: String);

    procedure HideCursor;
    procedure ShowCursor;

    Procedure TakeScreenshot(Filename: String);
    Function OnPoint(Coord: TCoordInt; Rect: TelRect): Boolean;

    function Run: Boolean;

    // SetFPS is not that precise at the moment, but if you want use a random fps limit
    // depending on your operating system and the current mood of your computer
    procedure SetFPS(FPS: Integer);
    function GetFPS: Integer;
end;

TelGameStates = class
  private
    FStates: TList;
  public
    constructor Create;

    procedure Add(Name: String);
    procedure Del(Name: String); Overload;
    procedure Del(Index: Integer); Overload;

    procedure SetGameState(Name: String); Overload;
    procedure SetGameState(Index: Integer); Overload;
    function GetGameState(Name: String): Boolean; Overload;
    function GetGameState(Index: Integer): Boolean; Overload;
end;


TelTimer = class
  private
    FActive: Boolean;
    FInterval: Integer;

    StartTick: Integer;

  public
    constructor Create; Overload;
    constructor Create(Interval: Integer); Overload;

    procedure SetActive(Active: Boolean);
    function GetActive: Boolean;

    procedure SetInterval(Interval: Integer);
    function GetInterval: Integer;

    function Action: Boolean;
    function SingleAction: Boolean;
end;

TQuality = (tqSolid, tqShaded, tqBlended);
TAlignment = (taNone, taLeft, taCenter, taRight);
TelFont = class
  private
    FAlpha: Byte;
    FUseTTF: Boolean;
    
    FFontInfo: TSFont_FontInfo;
    FSurface: PSDL_Surface;
    parentSurface: PSDL_Surface;
  public
    constructor Create; Overload;
    constructor Create(Surface: TelSurface); Overload;
    procedure Free;

    procedure Assign(Surface: TelSurface);

    procedure LoadFromFile(Filename: String);
    
    procedure SetTransparency(Color: TRGBColor); Overload;
    procedure SetTransparency(Coord: TCoordInt); Overload;
    
    procedure SetAlpha(Alpha: Byte);
    function GetAlpha: Byte;
    
    function GetWidth(Text: String): Integer;
    function GetHeight: Integer;
    
    procedure TextOut(X, Y: Integer; Text: String; TextAlignment: TAlignment = taNone);

    procedure SetColor(Color: TRGBColor);
    function GetColor: TRGBColor;
end;

TelSprite = class
  private
    parentSurface: PSDL_Surface;
    FTexture: PSDL_Surface;

    FImageType: String;

    FFrame, FMaxFrames: Integer;
    FAnim: TelRect;
    FName: String;
    FZ: Integer;
    FVisible: Boolean;

    FRect: TelRect;
    FCutRect: TelRect;
    FAlpha: Byte;
  public
    constructor Create; Overload;
    constructor Create(Surface: TelSurface); Overload;

    procedure Assign(Surface: TelSurface);

    function LoadFromFile(Filename: String): Boolean; Overload;
    function LoadFromFile(Filename: String; CutRect: TelRect): Boolean; Overload;
	
	procedure Generate;

    function GetImageType: String;
    function GetSDLSurface: PSDL_Surface;

    procedure CutImage(Rect: TelRect);
    function GetCutRect: TelRect;
	function GetRect: TelRect;

    procedure SetPosition(Coord: TCoordInt);
    procedure Move(Coord: TCoordInt);
    function GetPosition: TCoordInt;
    procedure SetZ(Z: Integer);
    function GetZ: Integer;

    procedure SetVisible(Visible: Boolean);
    function GetVisible: Boolean;

    procedure SetWidth(Width: Integer);
    function GetWidth: Integer;
    procedure SetHeight(Height: Integer);
    function GetHeight: Integer;
    function GetSurfaceHeight: Integer; // read-only
    function GetSurfaceWidth: Integer; // read-only

    procedure SetName(Name: String);
    function GetName: String;

    procedure SetFrame(Frame: Integer);
    function GetFrame: Integer;
    function GetMaxFrames: Integer;

    procedure SetTransparency(Color: TRGBColor); Overload;
    procedure SetTransparency(Coord: TCoordInt); Overload;

    procedure SetAlpha(Alpha: Byte);
    function GetAlpha: Byte;

    procedure Draw;

    procedure Animate(Timer: TelTimer; Loop: Boolean = true);

    function OnPoint(Coord: TCoordInt): Boolean;

    procedure Free;
end;

//TelSpriteList = class(TList)
//  protected
//    function Get(Index: Integer): TelSprite;
//    procedure Put(Index: Integer; Item : TelSprite);
//  public
//    property Items[Index: Integer]: TelSprite read Get write Put; default;
//end;

TelSpriteEngine = class
  private

  public
    UpdateRects: array of TelRect;
end;

TelSound = class
  private

  public

end;

function makeCol(R, G, B: Byte): TRGBColor;
function convCol(S: TelSurface; Color: TRGBColor): Cardinal; Overload;
function convCol(S: TelSurface; R, G, B: Byte): Cardinal; Overload;
function makeP2D(X, Y: Single): TCoord; Overload;
function makeP2D(X, Y: Integer): TCoordInt; Overload;
function makeRect(X, Y, W, H: Integer): TelRect;
function delayTime(FPS: Integer): UInt32;
function CompareZ(Item1, Item2: TelSprite): Integer;
function OnPoint(Coord: TCoordInt; Rect: TelRect): Boolean;
function GetFilenameExtension(Filename: String): String;

implementation

// Uh, those are some pretty functions
function makeCol(R, G, B: Byte): TRGBColor;
begin
  Result.R := R;
  Result.G := G;
  Result.B := B;
end;

function convCol(S: TelSurface; Color: TRGBColor): Cardinal; Overload;
begin
  Result := SDL_MapRGB(S.SDL_Surface.Format, Color.R, Color.G, Color.B);
end;

function convCol(S: TelSurface; R, G, B: Byte): Cardinal; Overload;
begin
  Result := SDL_MapRGB(S.SDL_Surface.Format, R, G, B);
end;

function makeP2D(X, Y: Single): TCoord; Overload;
begin
  Result.X := X;
  Result.Y := Y;
end;

function makeP2D(X, Y: Integer): TCoordInt; Overload;
begin
  Result.X := X;
  Result.Y := Y;
end;

function makeRect(X, Y, W, H: Integer): TelRect;
begin
  Result.X := X;
  Result.Y := Y;
  Result.W := W;
  Result.H := H;
end;

function delayTime(FPS: Integer): UInt32;
var prev_time, tick_interval, now: UInt32;
    delay_time: Byte;
begin
  prev_time := SDL_GetTicks;
  tick_interval := 1000 div FPS;
  now := SDL_GetTicks;
  delay_time := tick_interval - (now - prev_time);

  prev_time := now + delay_time;

  if delay_time > 0 then Result := delay_time
                    else Result := 0;
end;

function CompareZ(Item1, Item2 : TelSprite): Integer;
begin
  if Item1.GetZ < Item2.GetZ then Result := -1
  else if Item1.GetZ > Item2.GetZ then Result := 1
                                  else Result := 0;
end;

function OnPoint(Coord: TCoordInt; Rect: TelRect): Boolean;
begin
  if (Coord.X >= Rect.X) and
     (Coord.Y >= Rect.Y) and
     (Coord.X < (Rect.X + Rect.W)) and
     (Coord.Y < (Rect.Y + Rect.H)) then Result := true else Result := false;
end;

function GetFilenameExtension(Filename: String): String;
begin
  Result := UpperCase(Copy(Filename, Length(Filename) - 2, Length(Filename)));
end;

//
// TelSurface
//
//
constructor TelSurface.Create;
begin
  inherited Create;

  Cursor.X := -1;
  Cursor.Y := -1;

  FHideCursor := true;
  FRun := true;
  FFullscreenFlag := SDL_FULLSCREEN;
  
  FStartTick := 0;
  FCurTick := 0;
  FTickNow := 0;
  FFrames := 0;
  FFPS := 0;

  fnext_time := 0;
  ffps := 0;
  f__fps := 0;
end;

function TelSurface.Initialize(Width, Height, ColorBits: Integer; Fullscreen: Boolean): Boolean;
var VideoInfo: PSDL_VideoInfo;
begin
  Result := true;

  FWidth := Width;
  FHeight := Height;
  FFullscreen := Fullscreen;

  if SDL_Init(SDL_INIT_VIDEO or SDL_INIT_TIMER or SDL_INIT_AUDIO) <> 0 then
  begin
    Result := false;
    Finalize;
  end;

  FStartTick := SDL_GetTicks;
  
  VideoInfo := SDL_GetVideoInfo;

  if VideoInfo = nil then 
  begin
    Finalize;
  end;
  
  FFlags := SDL_DOUBLEBUF or SDL_HWPALETTE;
  
  if VideoInfo.hw_available <> 0  then FFlags := FFlags or SDL_HWSURFACE
                                  else FFlags := FFlags or SDL_SWSURFACE;

  SDL_WM_SetCaption(PChar('Elysion application'), nil);

  SetVideoMode(Width, Height, ColorBits, Fullscreen);

  if not Assigned(Self.SDL_Surface) then
  begin
    Result := false;
    Finalize;
  end;

  if FHideCursor then SDL_ShowCursor(0)
                 else SDL_ShowCursor(1);

end;

procedure TelSurface.SetVideoMode(Width, Height, ColorBits: Integer; Fullscreen: Boolean = False);
begin
  if Width <> FWidth then FWidth := Width;
  if Height <> FHeight then FHeight := Height;
  if ColorBits <> FColorBits then FColorBits := ColorBits;

  if Fullscreen then
  begin
    FGFlags := FFlags or FFullscreenFlag;
    Self.SDL_Surface := SDL_SetVideoMode(Width, Height, ColorBits, FGFlags);
  end else Self.SDL_Surface := SDL_SetVideoMode(Width, Height, ColorBits, FFlags);
end;

procedure TelSurface.Finalize;
begin
  if FFullscreen then SDL_WM_ToggleFullScreen(Self.SDL_Surface);
  SDL_FreeSurface(Self.SDL_Surface);
  SDL_Quit;
  Halt(0);
end;

procedure TelSurface.BeginScene;
var i: Integer;
begin


  if UseBackgroundImage then
    SDL_BlitSurface(FBackgroundImage, PSDLRect(0, 0, FBackgroundImage.w, FBackgroundImage.h), SDL_Surface, PSDLRect(0, 0, FBackgroundImage.w, FBackgroundImage.h))
  else
    SDL_FillRect(Self.SDL_Surface, Nil, convCol(Self, FBackgroundColor));
end;

procedure TelSurface.EndScene;
//var Rect: array[0..0] of TelRect;
begin
  SDL_Flip(Self.SDL_Surface);

  HandleEvents;
  
  //Rect[0] := makeRect(0, 0, 0, 0);
  //EndScene(Rect);
end;

procedure TelSurface.EndScene(UpdateRects: array of TelRect);
var i: Integer;
begin
  for i := 0 to High(UpdateRects) do
    SDL_UpdateRect(Self.SDL_Surface, UpdateRects[i].X, UpdateRects[i].Y, UpdateRects[i].W, UpdateRects[i].H);
  SDL_Flip(Self.SDL_Surface);

  HandleEvents;
  //SDL_Delay(delayTime(60));



end;

procedure TelSurface.SetFPS(FPS: Integer);
begin

end;

function TelSurface.GetFPS: Integer;
begin
  f__fps := f__fps + 1;
  fnow := SDL_GetTicks;
  if fnext_time <= fnow then
  begin
    fnext_time := fnow + 1000;
    ffps := f__fps;
    f__fps := 0;
  end;

  Result := ffps;

 // Result := FFPS;
end;

procedure TelSurface.Quit;
begin
  FRun := false;
end;

procedure TelSurface.SetBackgroundImage(Surface: PSDL_Surface);
begin
  UseBackgroundImage := true;
  FBackgroundImage := Surface;
end;

procedure TelSurface.SetBackgroundColor(Color: TRGBColor);
begin
  UseBackgroundImage := false;
  FBackgroundColor := Color;
end;

procedure TelSurface.SetCaption(Caption: String);
begin
  FCaption := PChar(Caption);
  SDL_WM_SetCaption(FCaption, FIcon);
end;

procedure TelSurface.SetIcon(Icon: String);
var TMP_Icon: PSDL_Surface;
    Directory: String;
begin
  if Icon <> '' then
  begin
    Directory := ExtractFilePath(ParamStr(0));

    if FileExists(Icon) then
    begin
      TMP_Icon := IMG_Load(PChar(Directory+Icon));
      FIcon := PChar(Icon);
      SDL_WM_SetIcon(TMP_Icon, 0);
    end;

  end;
  
  if TMP_Icon <> nil then SDL_FreeSurface(TMP_Icon);
end;

procedure TelSurface.HideCursor;
begin
  FHideCursor := true;
  SDL_ShowCursor(0);
end;

procedure TelSurface.ShowCursor;
begin
  FHideCursor := false;
  SDL_ShowCursor(1);
end;

procedure TelSurface.TakeScreenshot(Filename: String);
begin
  SDL_SaveBMP(Self.SDL_Surface, PChar(Filename));
end;

function TelSurface.OnPoint(Coord: TCoordInt; Rect: TelRect): Boolean;
begin
  if (Coord.X >= Rect.x) and
     (Coord.X >= Rect.y) and
     (Coord.X < (Rect.x + Rect.w)) and
     (Coord.Y < (Rect.y + Rect.h)) then Result := true else Result := false;
end;

function TelSurface.Run: Boolean;
begin





 (* FTickNow := FCurTick - FStartTick;

  if FTickNow >= 1000 then
  begin
    if FFrames > 0 then FFPS := FFrames;
    FFrames := 0;
    FCurTick := 0;
    FFrameCount := FFrameCount + 1;
  end else
  begin
    FFrames := FFrames + 1;
    FCurTick := (SDL_GetTicks - FFrameCount * 1000);
  end;  *)

  if FRun then Result := true
          else Result := false;
end;

procedure TelSurface.SetDelayTime;
begin

end;

procedure TelSurface.HandleEvents;
var i: Integer;
begin
   for i := 0 to High(KeyUp) do
  begin
    KeyUp[i] := false;
  end;

  for i := 0 to High(MouseButtonUp) do
  begin
    MouseButtonUp[i] := false;
  end;

  while (SDL_PollEvent(@FEvent) = 1) do
  begin


    case FEvent.type_ of
    SDL_QUITEV:
       begin
         FRun := false;
       end;

    SDL_KEYUP:
        begin
          KeyDown[FEvent.key.keysym.sym] := false;
          KeyUp[FEvent.key.keysym.sym] := true;
        End;

    SDL_KEYDOWN:
        begin
          KeyDown[FEvent.key.keysym.sym] := true;
          KeyUp[FEvent.key.keysym.sym] := false;
        end;

    SDL_MOUSEMOTION:
        begin
          SDL_GetMouseState(Cursor.X, Cursor.Y);
        end;
 
    SDL_MOUSEBUTTONDOWN:
        begin
          MouseButtonDown[FEvent.button.button] := true;
          MouseButtonUp[FEvent.button.button] := false;
        end;

    SDL_MOUSEBUTTONUP:
        begin
          MouseButtonDown[FEvent.button.button] := false;
          MouseButtonUp[FEvent.button.button] := true;
        end;

    end;


  end;
end;

//
// TelGameStates
//
constructor TelGameStates.Create;
begin
  FStates := TList.Create;
end;

procedure TelGameStates.Add(Name: string);
begin

end;

procedure TelGameStates.Del(Name: string);
begin
 
end;

procedure TelGameStates.Del(Index: Integer);
begin

end;

procedure TelGameStates.SetGameState(Name: string);
begin

end;

procedure TelGameStates.SetGameState(Index: Integer);
begin

end;

function TelGameStates.GetGameState(Name: string): Boolean;
begin

end;

function TelGameStates.GetGameState(Index: Integer): Boolean; 
begin

end;

//
// TelTimer
//
constructor TelTimer.Create;
begin
  FInterval := 0;
end;

constructor TelTimer.Create(Interval: Integer);
begin
  SetInterval(Interval);
end;

procedure TelTimer.SetActive(Active: Boolean);
begin
  FActive := Active;
  StartTick := SDL_GetTicks;
end;

function TelTimer.GetActive: Boolean;
begin
  Result := FActive;
end;

procedure TelTimer.SetInterval(Interval: Integer);
begin
  FInterval := Interval;
end;

function TelTimer.GetInterval: Integer;
begin
	Result := FInterval;
end;

//
// Action
// To be used like an On-/Off-Switch
function TelTimer.Action: Boolean;
begin
  Result := false;

  if Self.GetActive then
  begin
	  if FInterval <> 0 then
	  begin
      if (((SDL_GetTicks - StartTick) div FInterval) mod 2) = 0 then Result := false
                                                                else Result := true;
	  end;
  end;
end;

//
// SingleAction
// A bit shaky
// Supposed to SingleAction = true every FInterval ms
// TODO: Idea: Using tolerance < SingleAction > tolerance
function TelTimer.SingleAction: Boolean;
var diff: Integer;
begin
  Result := false;

  if Self.GetActive then
  begin
    if FInterval <> 0 then
    begin
      diff := SDL_GetTicks - StartTick;

      if ((diff mod FInterval) = 0) then Result := true
                                    else Result := false;
    end;
  end;
end;

constructor TelFont.Create;
begin
  inherited Create;
  
  FAlpha := 255;
end;

constructor TelFont.Create(Surface: TelSurface);
begin
  inherited Create;
  Assign(Surface);
end;

procedure TelFont.Free;
begin

end;

procedure TelFont.Assign(Surface: TelSurface);
begin
  parentSurface := Surface.SDL_Surface;
end;

procedure TelFont.LoadFromFile(Filename: String);
var length_str, i: Integer;
	tmp: String;
  Directory: String;
begin
  length_str := Length(Filename);

  Directory := ExtractFilePath(ParamStr(0));
  
  (*for i := length_str - 2 to length_str do
  begin
    tmp[i - length_str - 2] := Filename[i];
  end;
  
  if tmp = LowerCase('ttf') then
  begin
    FUseTTF := true;
    
  end else
  begin
    FUseTTF := false;
                            *)
    if FileExists(Filename) then
    begin
      FFontInfo.Surface := IMG_Load(PChar(Directory + Filename));
      SFont_InitFont2(@FFontInfo);
    end;
  //end;
  
  
end;

procedure TelFont.SetTransparency(Color: TRGBColor);
begin
  if FFontInfo.Surface <> nil then
    SDL_SetColorKey(FFontInfo.Surface, SDL_SRCCOLORKEY or SDL_RLEACCEL or SDL_HWACCEL,
      SDL_MapRGB(FFontInfo.Surface.Format, Color.R, Color.G, Color.B));
end;

procedure TelFont.SetTransparency(Coord: TCoordInt);
begin
  if FFontInfo.Surface <> nil then
    SDL_SetColorKey(FFontInfo.Surface, SDL_SRCCOLORKEY or SDL_RLEACCEL or SDL_HWACCEL,
      SDL_GetPixel(FFontInfo.Surface, Coord.X, Coord.Y));
end;
    
procedure TelFont.SetAlpha(Alpha: Byte);
begin
  FAlpha := Alpha;
end;

function TelFont.GetAlpha: Byte;
begin
  Result := FAlpha;
end;
    
function TelFont.GetWidth(Text: String): Integer;
begin
  Result := SFont_TextWidth2(@FFontInfo, PChar(Text));
end;
    
function TelFont.GetHeight: Integer;
begin
  Result := FFontInfo.Surface.h;
end;

procedure TelFont.TextOut(X, Y: Integer; Text: String; TextAlignment: TAlignment = taNone);
begin
  if FAlpha <> 255 then SDL_SetAlpha(FFontInfo.Surface, SDL_SRCALPHA, FAlpha);

  case TextAlignment of
    taNone: SFont_Write2(parentSurface, @FFontInfo, X, Y, PChar(Text));
    taLeft: SFont_Write2(parentSurface, @FFontInfo, 0, Y, PChar(Text));
    taCenter: SFont_Write2(parentSurface, @FFontInfo, ((parentSurface.w - GetWidth(Text)) div 2), Y, PChar(Text));
    taRight: SFont_Write2(parentSurface, @FFontInfo, (parentSurface.w - GetWidth(Text)), Y, PChar(Text));
  end;
end;

procedure TelFont.SetColor(Color: TRGBColor);
begin

end;

function TelFont.GetColor: TRGBColor;
begin

end;


constructor TelSprite.Create;
begin
   FCutRect.X := 0;
   FCutRect.Y := 0;
   FCutRect.W := 0;
   FCutRect.H := 0;

   FRect.X := 0;
   FRect.Y := 0;
   FRect.W := 0;
   FRect.H := 0;

   FAlpha := 255;
   FFrame := 0;
   FVisible := true;
end;

constructor TelSprite.Create(Surface: TelSurface);
begin
  Assign(Surface);
end;

procedure TelSprite.Assign(Surface: TelSurface);
begin
  parentSurface := Surface.SDL_Surface;
end;

function TelSprite.LoadFromFile(Filename: String): Boolean;
begin
  Result := LoadFromFile(Filename, makeRect(0, 0, 0, 0));
end;

function TelSprite.LoadFromFile(Filename: String; CutRect: TelRect): Boolean;
var Directory: String;
begin
  Result := false;

  FAnim.X := 0;
  FAnim.Y := 0;

  FCutRect := CutRect;

  if Filename <> '' then
  begin
    Directory := ExtractFilePath(ParamStr(0));
	fImageType := GetFilenameExtension(Filename);

    if FileExists(Directory+FileName) then
    begin
      FTexture := IMG_Load(PChar(Directory+Filename));

      if FCutRect.W = 0 then FCutRect.W := FTexture.w;
      if FCutRect.H = 0 then FCutRect.H := FTexture.h;

      CutImage(FCutRect);

      FAnim.W := GetSurfaceWidth div FCutRect.W;
      FAnim.H := GetSurfaceHeight div FCutRect.H;

      FMaxFrames := FAnim.W * FAnim.H;
    end;
  end;
end;

procedure TelSprite.Generate;
begin
  if fImageType = 'BMP' then fTexture := SDL_DisplayFormat(fTexture)
						else fTexture := SDL_DisplayFormatAlpha(fTexture);
end;

function TelSprite.GetSDLSurface: PSDL_Surface;
begin
  Result := FTexture;
end;

function TelSprite.GetImageType: String;
begin
  Result := fImageType;
end;

procedure TelSprite.CutImage(Rect: TelRect);
begin
  FCutRect := Rect;
  FRect.W := FCutRect.W;
  FRect.H := FCutRect.H;
end;

function TelSprite.GetCutRect: TelRect;
begin
  Result := FCutRect;
end;

function TelSprite.GetRect: TelRect;
begin
  Result := FRect;
end;

procedure TelSprite.SetPosition(Coord: TCoordInt);
begin
  FRect.X := Coord.X;
  FRect.Y := Coord.Y;
end;

procedure TelSprite.Move(Coord: TCoordInt);
begin
  FRect.X := FRect.X + Coord.X;
  FRect.Y := FRect.Y + Coord.Y;
end;

function TelSprite.GetPosition: TCoordInt;
begin
  Result.X := FRect.X;
  Result.Y := FRect.Y;
end;

procedure TelSprite.SetZ(Z: Integer);
begin
  FZ := Z;
end;

function TelSprite.GetZ;
begin
  Result := FZ;
end;

procedure TelSprite.SetVisible(Visible: Boolean);
begin
  FVisible := Visible;
end;

function TelSprite.GetVisible: Boolean;
begin
  Result := FVisible;
end;

procedure TelSprite.SetWidth(Width: Integer);
begin
  FRect.W := Width;
end;

function TelSprite.GetWidth: Integer;
begin
  Result := FRect.W;
end;

procedure TelSprite.SetHeight(Height: Integer);
begin
  FRect.H := Height;
end;

function TelSprite.GetHeight: Integer;
begin
  Result := FRect.H;
end;

function TelSprite.GetSurfaceWidth: Integer;
begin
  Result := FTexture.w;
end;

function TelSprite.GetSurfaceHeight: Integer;
begin
  Result := FTexture.h;
end;

procedure TelSprite.SetName(Name: string);
begin
  FName := Name;
end;

function TelSprite.GetName: String;
begin
  Result := FName;
end;

procedure TelSprite.SetFrame(Frame: Integer);
begin
   if Frame <= FMaxFrames then
   begin
     FFrame := Frame - 1;
     if FFrame < 0 then FFrame := 0;


     if FAnim.H = 1 then FAnim.X := FFrame
     else
     begin
       FAnim.Y := FAnim.H - 1;
       FAnim.X := FFrame - FAnim.H;
     end;

     CutImage(makeRect(FAnim.X * FCutRect.W, FAnim.Y * FCutRect.H, FCutRect.W, FCutRect.H));
   end;
end;

function TelSprite.GetFrame: Integer;
begin
  Result := FFrame;
end;

function TelSprite.GetMaxFrames: Integer;
begin
  Result := FMaxFrames;
end;

procedure TelSprite.SetTransparency(Color: TRGBColor);
begin
  if FTexture <> Nil then
    SDL_SetColorKey(FTexture, SDL_SRCCOLORKEY or SDL_RLEACCEL or SDL_HWACCEL,
      SDL_MapRGB(FTexture.Format, Color.R, Color.G, Color.B));
end;

procedure TelSprite.SetTransparency(Coord: TCoordInt);
begin
  if FTexture <> Nil then
    SDL_SetColorKey(FTexture, SDL_SRCCOLORKEY or SDL_RLEACCEL or SDL_HWACCEL,
      SDL_GetPixel(FTexture, Coord.X, Coord.Y));
end;

procedure TelSprite.SetAlpha(Alpha: Byte);
begin
  FAlpha := Alpha;
end;

function TelSprite.GetAlpha: Byte;
begin
  Result := FAlpha;
end;

procedure TelSprite.Draw;
begin
  if FVisible then
  begin
    if FAlpha <> 255 then SDL_SetAlpha(FTexture, SDL_SRCALPHA, FAlpha)
                     else SDL_SetAlpha(FTexture, SDL_SRCALPHA, 255);

    if (FCutRect.W <> FRect.W) or (FCutRect.H <> FRect.H) then
      SDL_BlitSurface(SDL_ScaleSurfaceRect(FTexture, FCutRect.X, FCutRect.Y, FCutRect.W, FCutRect.H, FRect.W, FRect.H),
                      PSDLRect(FCutRect.X, FCutRect.Y, FRect.W, FRect.H),
                      parentSurface, PSDLRect(FCutRect.X, FCutRect.Y, FRect.W, FRect.H))
    else
      SDL_BlitSurface(FTexture, PSDLRect(FCutRect.X, FCutRect.Y, FCutRect.W, FCutRect.H),
                              parentSurface, PSDLRect(FRect.X, FRect.Y, FCutRect.W, FCutRect.H));
  end;
end;

procedure TelSprite.Animate(Timer: TelTimer; Loop: Boolean = True);
var tmp_frame: Integer;
begin
  if Timer.SingleAction then FFrame := FFrame + 1;
  if Loop then if GetFrame = GetMaxFrames then FFrame := 0;

  Self.SetFrame(FFrame);
end;

function TelSprite.OnPoint(Coord: TCoordInt): Boolean;
begin
  if (Coord.X >= FRect.X) and
     (Coord.Y >= FRect.Y) and
     (Coord.X < (FRect.X + FRect.W)) and
     (Coord.Y < (FRect.Y + FRect.H)) then Result := true else Result := false;
end;

procedure TelSprite.Free;
begin
  if FTexture <> nil then SDL_FreeSurface(FTexture);
end;

end.
