(*
   Elysion Frameworks
   Branch: Legacy
   Version: 09-04 "Bunny"

   Copyright (c) 2009 Freeze.Dev (Johannes Stein and contributors)

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation
  files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use,
  copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  OTHER DEALINGS IN THE SOFTWARE.

  Thanks to:
  Sam Lantinga     - The father of SDL
  Dominique Louis  - Who took the time and translated SDL to Pascal
  Robrt Kisnmeth - Thanks a lot for the Sprite Engine. Helped me a lot
                     setting up TelSpriteEngine and including Z sorting
                     Nagyon ksznm.
  Sven Rech        -

  http://www.freeze-dev.de
  info@freeze-dev.de

  Please, just use it if you are extremely sure about it. Actually, I created
  this for myself. There could be a lot of API changes in the future.

  *)

unit ElysionLegacy;

{$IFDEF FPC}
  {$MESSAGE NOTE 'ElysionLegacy can and should be configured to your needs.'}
  {$MESSAGE NOTE 'For information have a look at the documentation.'}
{$ENDIF}

// To use PNGs, JPGs, GIFs, etc. instead of just BMPs
{$DEFINE USE_SDL_IMAGE}

// To use audio
{$DEFINE USE_SDL_MIXER}

// To use truetype fonts
{$DEFINE USE_SDL_TTF}

// To use networking capabilities
{.$DEFINE USE_SDL_NET}

// If you don't want to use SFont
{$DEFINE NO_SFONT}

{$IFDEF FPC}
	{$MODE DELPHI}
{$ENDIF}

interface

uses SDL,
     {$IFDEF USE_SDL_IMAGE} SDL_Image, {$ENDIF}
	 {$IFDEF USE_SDL_MIXER} SDL_Mixer, {$ENDIF}
	 {$IFDEF USE_SDL_TTF}   SDL_TTF,   {$ENDIF}
	 {$IFDEF USE_SDL_GFX}   SDL_gfx,   {$ENDIF}
     {$IFDEF USE_SDL_NET}   SDL_net    {$ENDIF}
     {$IFNDEF NO_SFONT}     SFont,     {$ENDIF}
	 Logger,
     SDLUtils,
     SysUtils,
     Classes;

const

{$I keymap.inc}

type

// TelfPoint: 
TelfPoint = record
  X, Y: Single;
end;

// TelPoint: 
TelPoint = record
  X, Y: Integer;
end;

// TelVector: 
TelVector = record
  X, Y, Z: Integer;
end;

// TelColor: Stores a RGB color
TelColor = Record
    R, G, B: Byte;
end;

// TelRGBAColor: Stores a RGB color plus alpha value
TelRGBAColor = Record
  R, G, B, Alpha: Byte;
end;

// TelRect: 
TelRect = Record
  X, Y, W, H: Integer;
end;


TelInitFlag = (ifSoftware, ifHardware, ifDoubleBuf, ifStandard);

TelAlignment = (taNone, taLeft, taCenter, taRight);

TelCursor = class
  private
	fImageType: String;
	
	fAlpha: Byte;
	
	fSurface: PSDL_Surface;
	parentSurface: PSDL_Surface;
  public
    Position: TelPoint;
	
	// Do not use TelCursor directly!!! just via TelSurface
    constructor Create(Surface: PSDL_Surface);
	
	procedure Free;
	
	function GetSDLSurface: PSDL_Surface;
	
	// Load an image from a file
	function LoadFromFile(const Filename: String): Boolean;
	
	//
	procedure SetTransparency(Coord: TelPoint); Overload;
	
	//
	procedure SetTransparency(Color: TelColor); Overload;
	
	function GetImageType: String;
	
	// Set alpha value
	procedure SetAlpha(Alpha: Byte);
	
	// Get alpha value
	function GetAlpha: Byte;
	
	procedure SetPosition(Coord: TelPoint);
	
	// Draw Cursor
	procedure Draw;
	
  published
    property Alpha: Byte read GetAlpha write SetAlpha;
	property ImageType: String read GetImageType;
    //property Position: TelPoint read fPosition;
end;

TelSurface = class
  private
	fHeight, fWidth, fColorBits: Integer;
	fFullscreen: Boolean;
	fHideCursor: Boolean;

	fRun: Boolean;
    fFlags, fGFlags: Cardinal;
    fFullscreenFlag: Cardinal;

    UseBackgroundImage: Boolean;
    fBackgroundColor: TelColor;
    fBackgroundImage: PSDL_Surface;

	fUseLog: Boolean;

    FEvent: TSDL_Event;
    
    FCaption, FIcon: PChar;

    FStartTick, FCurTick, FTickNow: UInt32;
    FFrames, FFrameCount: UInt32;

    fnext_time: UInt32;
    ffps: Integer;
    f__fps: Integer;
    fnow: UInt32;
	
	fCursor: TelCursor;

    procedure SetDelayTime;
    procedure HandleEvents;
	procedure DrawCursor;
  public
    // You can access the SDL surface directly if you wish
    SDL_Surface: PSDL_Surface;

	// True, if key is pressed down
    KeyDown: array[0..K_EURO] of Boolean;
	
	// True, if key is pressed up again
    KeyUp: array[0..K_EURO] of Boolean;
	
    Cursor: TelPoint;
	
	// True, if mouse button is pressed down
    MouseButtonDown: array[BUTTON_LEFT..BUTTON_WHEELDOWN] of Boolean;
	
	// True, if mouse button is pressed up again
    MouseButtonUp: array[BUTTON_LEFT..BUTTON_WHEELDOWN] of Boolean;

	// Creates this class
    constructor Create;
	
    // Initialize the surface. First thing you need to do after creating the surface
	// Width, Height: Width and height of the surface
	// ColorBits: Sets 
	// Fullscreen: If true, 
	// InitFlag (optional): Specify if you want to use software or hardware-accelerated surface
	//						ifStandard means it will be detected automatically what's best
	//						Warning: Just use this if you know what you're doing.
	//								 There is no failsafe, if you want to use ifHardware and your
	//								 computer can't handle a hardware-accelerated surface you
	//								 will most likely get an access violation.
    function Initialize(Width, Height, ColorBits: Integer; Fullscreen: Boolean; InitFlag: TelInitFlag = ifStandard): Boolean;
	
	// Sets video mode for the surface. Use this if you want to change the resolution in-game.
	// Same parameter as Initialize except for InitFlag
    procedure SetVideoMode(Width, Height, ColorBits: Integer; Fullscreen: Boolean = False);

	// Finalizes the surface
    procedure Finalize;

	// Use BeginScene first thing in the game loop
    procedure BeginScene;
	
	// Use EndScene last thing before the end of the game loop
    procedure EndScene; Overload;

    // Expert function: Use rects to update surface
    // Improves performance if software surface is used
    procedure EndScene(UpdateRects: array of TelRect); Overload;

    procedure Quit;

    // Use background image instead of a color
    procedure SetBackgroundImage(const Filename: String);
    procedure SetBackgroundColor(const Color: TelColor);

	// Sets the window title
    procedure SetCaption(Caption: String);
	
	// Sets a window icon
    procedure SetIcon(Icon: String);

	// Hide the standard black/white cursor. Normally turned on
    procedure HideCursor;
	
	// Show the cursor
    procedure ShowCursor;

	// Saves the surface into an image
    procedure TakeScreenshot(const Filename: String);
	
	//
	procedure SetLogger(const Filename: String);
	
	//
	function GetLoggerStatus: Boolean;
	
	//
    function OnPoint(Rect: TelRect): Boolean;

	//
    function Run: Boolean;

    // SetFPS is not that precise at the moment, but if you want use a random fps limit
    // depending on your operating system and the current mood of your computer
    procedure SetFPS(FPS: Integer);
    function GetFPS: Integer;
	
    published
	  property Cursor: TelCursor read fCursor;
end;

TelObject = class
  private
    fName: String;
	
  public
    constructor Create;
	
	// Set Name for this object
    procedure SetName(Name: String);
	
	// Get name from this object
	function GetName: String;
end;

TelGraphicObject = class(TelObject)
  private
    // parentSurface:
    parentSurface: PSDL_Surface;
	
  public
    constructor Create; Overload;
	
	// Create 
	constructor Create(Surface: TelSurface); Overload;
	procedure Free;

    // Assigns this object to a TelSurface
	procedure Assign(Surface: TelSurface); Overload;

    // Expert method: Assigns this object to a specific SDL_Surface
    procedure Assign(Surface: PSDL_Surface); Overload;


	function GetParentSurface: PSDL_Surface;
end;

TelGameStates = class
  private
    fStates: TList;
  public
    constructor Create;

	// 
    procedure Add(Name: String);
    procedure Del(Name: String); Overload;
    procedure Del(Index: Integer); Overload;

    procedure SetGameState(Name: String); Overload;
    procedure SetGameState(Index: Integer); Overload;
    function GetGameState(Name: String): Boolean; Overload;
    function GetGameState(Index: Integer): Boolean; Overload;
	function GetGameState: String; Overload;
end;


TelTimer = class(TelObject)
  private
    FActive: Boolean;
    FInterval: Integer;

    StartTick: Integer;

  public
    constructor Create; Overload;
    constructor Create(Interval: Integer); Overload;

    procedure SetActive(Active: Boolean);
    function GetActive: Boolean;

    procedure SetInterval(Interval: Integer);
    function GetInterval: Integer;

    function Action: Boolean;
    function SingleAction: Boolean;
end;


{$IFNDEF NO_SFONT}
TelFont = class(TelGraphicObject)
  private
    fFontInfo: TSFont_FontInfo;
  public
    procedure LoadFromFile(const Filename: String);
	
	procedure SetTransparency(Color: TelColor); Overload;
	procedure SetTransparency(Coord: TelPoint); Overload;
	
	function GetWidth(Text: String): Integer;
	function GetHeight: Integer;
	
	procedure TextOut(Coord: TelPoint; Text: String; TextAlignment: TelAlignment = taNone);
end;
{$ENDIF}

{$IFDEF USE_SDL_TTF}
TelQuality = (tqSolid, tqShaded, tqBlended);

TelTrueTypeFont = class(TelGraphicObject)
  private
  
  public
    procedure LoadFromFile(const Filename: String; Size: Integer);
	
	function GetWidth(Text: String): Integer;
	function GetHeight: Integer;
	
	procedure TextOut(Coord: TelPoint; Text: String; TextQuality: TelQuality = tqSolid; TextAlignment: TelAlignment = taNone);
end;
{$ENDIF}

TelPrimitive = class(TelGraphicObject)
  public
    procedure DrawRect(Rect: TelRect; Color: TelColor);
    procedure DrawLine(Origin, Destination: TelPoint; Color: TelColor);
end;

TelLayerType = (ltSprite, ltImageFont, ltTrueTypeFont);

TelAnimation = class
  private
    fFrame, fMaxFrames: Integer;
    fAnim, parentRect: TelRect;
  public
    constructor Create(CutRect, Rect: TelRect);
	
	procedure GetRect: TelRect;
	
	procedure SetFrame(Frame: Integer);
    function GetFrame: Integer;
    function GetMaxFrames: Integer;
	
	procedure SetAnimation(CutRect, Rect: TelRect);
  published
    property Frame: Integer read GetFrame write SetFrame;
    property MaxFrames: Integer read GetMaxFrames;
end;

TelLayer = class(TelGraphicObject)
  private    
    FZ: Integer;

    fRect: TelRect;
	fAlpha: Byte;
	
	fType: TelLayerType;
  public
    Animation: TelAnimation;
  
    function LoadFromFile(Filename: String): Boolean; Overload;
    function LoadFromFile(Filename: String; CutRect: TelRect): Boolean; Overload;
	
	procedure SetLayerType(const LayerType: TelLayerType);
	function GetLayerType: TelLayerType;

    procedure CutImage(Rect: TelRect);
    function GetCutRect: TelRect;

    procedure SetZ(Z: Integer);
    function GetZ: Integer;


    procedure SetWidth(Width: Integer);
    function GetWidth: Integer;
    procedure SetHeight(Height: Integer);
    function GetHeight: Integer;
    function GetSurfaceHeight: Integer; // read-only
    function GetSurfaceWidth: Integer; // read-only

    procedure SetName(Name: String);
    function GetName: String;

    // Get SDL surface to work with if you want to
    function GetSDLSurface: PSDL_Surface;

    // Set Transparency
	procedure SetTransparency(Color: TelColor); Overload;
	procedure SetTransparency(Coord: TelPoint); Overload;
	
	// Set alpha value
	procedure SetAlpha(Alpha: Byte);
	// Get alpha value
	function GetAlpha: Byte;
	
	procedure SetColor(Color: TelColor);
	function GetColor: TelColor;
	
	// Set position
	procedure SetPosition(Coord: TelPoint);
	// Get position
	function GetPosition: TelPoint;
	
	procedure SetRect(Rect: TelRect);
	function GetRect: TelRect;
	
	procedure Move(Coord: TelPoint);
	
	procedure SetVisible(Visible: Boolean);
	function GetVisible: Boolean;

    {$IFDEF USE_SDL_gfx}
	procedure SetRotation(Angle: Single);
	function GetRotation;
	{$ENDIF}

    procedure Draw;

    procedure Animate(Timer: TelTimer; Loop: Boolean = true);

    function OnPoint(Coord: TelPoint): Boolean;

    procedure Free;
  protected
    property LayerType: TelLayerType read GetLayerType write SetLayerType;
	
	property Alpha: Byte read GetAlpha write SetAlpha;
end;

TelSpriteList = class(TList)
  protected
    function Get(Index: Integer): TelLayer;
    procedure Put(Index: Integer; Item : TelLayer);
  public
    property Items[Index: Integer]: TelLayer read Get write Put; default;
end;

TelSpriteEngine = class(TelGraphicObject)
  private
    parentSurface: PSDL_Surface;
	
  public
    Sprite: TelSpriteList;
	NumberOfRects: Integer;
    UpdateRects: array of TelRect;
	

	procedure Add(Item: TelLayer);
	procedure Del(Item: TelLayer);
end;

{$IFDEF USE_SDL_MIXER}
TelMusic = class
  private
  
  public
    constructor Create; overload;
    constructor Create(Filename: String); overload;
    procedure Free;

    procedure LoadFromFile(Filename: String);
    procedure Play;
    procedure Stop;
end;

TelSound = class
  private
    fResume: Boolean;
    fSound: PMix_Chunk;
  public
    constructor Create; overload;
    constructor Create(Filename: String); overload;
    procedure Free;

    procedure LoadFromFile(Filename: String);
    procedure Play;
    procedure Stop;
end;
{$ENDIF}

// 
function makeCol(R, G, B: Byte): TelColor; Overload;
function makeCol(R, G, B, A: Byte): TelRGBAColor; Overload;

// Converts color into Cardinal for SDL to use
function convCol(S: TelSurface; Color: TelColor): Cardinal; Overload;
function convCol(S: TelSurface; R, G, B: Byte): Cardinal; Overload;
function convCol(S: PSDL_Surface; Color: TelColor): Cardinal; Overload;
function convCol(S: PSDL_Surface; R, G, B: Byte): Cardinal; Overload;
function makeP2D(X, Y: Single): TelfPoint; Overload;
function makeP2D(X, Y: Integer): TelPoint; Overload;
function makeRect(X, Y, W, H: Integer): TelRect;

// Checks if a point is within a rect
function OnPoint(Coord: TelPoint; Rect: TelRect): Boolean;

function delayTime(FPS: Integer): UInt32;
function CompareZ(Item1, Item2: TelLayer): Integer;
procedure AutoAssign(Surface: TelSurface);

var UseAutoAssign: Boolean = False; // 
    UeberSurface: PSDL_Surface; // This is berSurface; It is above anything else ;)

implementation

// Uh, those are some pretty functions
function makeCol(R, G, B: Byte): TelColor;
begin
  Result.R := R;
  Result.G := G;
  Result.B := B;
end;

function makeCol(R, G, B, A: Byte): TelRGBAColor;
begin
  Result.R := R;
  Result.G := G;
  Result.B := B;
  Result.Alpha := A;
end;

function convCol(S: TelSurface; Color: TelColor): Cardinal; Overload;
begin
  Result := SDL_MapRGB(S.SDL_Surface.Format, Color.R, Color.G, Color.B);
end;

function convCol(S: TelSurface; R, G, B: Byte): Cardinal; Overload;
begin
  Result := SDL_MapRGB(S.SDL_Surface.Format, R, G, B);
end;

function convCol(S: PSDL_Surface; Color: TelColor): Cardinal; Overload;
begin
  Result := SDL_MapRGB(S.Format, Color.R, Color.G, Color.B);
end;

function convCol(S: PSDL_Surface; R, G, B: Byte): Cardinal; Overload;
begin
  Result := SDL_MapRGB(S.Format, R, G, B);
end;

function makeP2D(X, Y: Single): TelfPoint; Overload;
begin
  Result.X := X;
  Result.Y := Y;
end;

function makeP2D(X, Y: Integer): TelPoint; Overload;
begin
  Result.X := X;
  Result.Y := Y;
end;

function makeRect(X, Y, W, H: Integer): TelRect;
begin
  Result.X := X;
  Result.Y := Y;
  Result.W := W;
  Result.H := H;
end;

function OnPoint(Coord: TelPoint; Rect: TelRect): Boolean;
begin
  if (Coord.X >= Rect.X) and
     (Coord.Y >= Rect.Y) and
     (Coord.X < (Rect.X + Rect.W)) and
     (Coord.Y < (Rect.Y + Rect.H)) then Result := true else Result := false;
end;

procedure AutoAssign(Surface: TelSurface);
begin
  UseAutoAssign := true;
  UeberSurface := Surface.SDL_Surface;
end;

function delayTime(FPS: Integer): UInt32;
var prev_time, tick_interval, now: UInt32;
    delay_time: Byte;
begin
  prev_time := SDL_GetTicks;
  tick_interval := 1000 div FPS;
  now := SDL_GetTicks;
  delay_time := tick_interval - (now - prev_time);

  prev_time := now + delay_time;

  if delay_time > 0 then Result := delay_time
                    else Result := 0;
end;

function CompareZ(Item1, Item2 : TelLayer): Integer;
begin
  if Item1.GetZ < Item2.GetZ then Result := -1
  else if Item1.GetZ > Item2.GetZ then Result := 1
                                  else Result := 0;
end;


//
// TelCursor
//

constructor TelCursor.Create(Surface: PSDL_Surface);
begin
  inherited Create;
  
  parentSurface := Surface;
end;

procedure TelCursor.Free;
begin
  inherited Free;
  
  if parentSurface <> nil then SDL_FreeSurface(parentSurface);
end;

function TelCursor.GetSDLSurface: PSDL_Surface;
begin
  Result := fSurface;
end;

function TelCursor.LoadFromFile(const Filename: String): Boolean;
var Directory: String;
begin
  Result := false;

  if Filename <> '' then
  begin
    Directory := ExtractFilePath(ParamStr(0));

    if FileExists(Directory+FileName) then
    begin
	  fImageType := UpperCase(copy(Filename, 1, Pos('.', Filename)-1));
	
      {$IFDEF USE_SDL_IMAGE}
      fSurface := IMG_Load(PChar(Directory+Filename));
      {$ELSE}
      fSurface := SDL_LoadBMP(PChar(Directory+Filename));
      {$ENDIF}
      
	  if fSurface = nil then
	  begin
	  
	  end else Result := true;
      
    end else 
	begin
	
	end;
  end;
  
end;
	
procedure TelCursor.SetTransparency(Coord: TelPoint);
begin

end;

procedure TelCursor.SetTransparency(Color: TelColor);
begin

end;
	
procedure TelCursor.SetAlpha(Alpha: Byte);
begin
  fAlpha := Alpha;
end;
	
function TelCursor.GetAlpha: Byte;
begin
  Result := fAlpha;
end;
	
procedure TelCursor.Draw;
begin
  if fSurface <> nil then
  begin
    if FAlpha <> 255 then SDL_SetAlpha(fSurface, SDL_SRCALPHA, FAlpha)
                     else SDL_SetAlpha(fSurface, SDL_SRCALPHA, 255);

	{$IFDEF USE_SDL_IMAGE}
	fSurface := SDL_DisplayFormatAlpha(fSurface);
	{$ELSE}
	fSurface := SDL_DisplayFormat(fSurface);
	{$ENDIF}
	

    //if (FCutRect.W <> FRect.W) or (FCutRect.H <> FRect.H) then
    //  SDL_ZoomSurface(FTexture, PSDLRect(FCutRect.X, FCutRect.Y, FCutRect.W, FCutRect.H),
    //                          parentSurface, PSDLRect(FRect.X, FRect.Y, FRect.W, FRect.H))
    //else  
      SDL_BlitSurface(fSurface, PSDLRect(FCutRect.X, FCutRect.Y, FCutRect.W, FCutRect.H),
                              parentSurface, PSDLRect(FRect.X, FRect.Y, FCutRect.W, FCutRect.H));
                              
  end;
end;
	
procedure TelCursor.SetPosition(Coord: TelPoint);
begin
  SDL_WarpMouse(fPosition.X, fPosition.Y);
end;

//
// TelSurface
//
//
constructor TelSurface.Create;
begin
  inherited Create;

  FHideCursor := true;
  FRun := true;
  FFullscreenFlag := SDL_FULLSCREEN;
  
  fUseLog := false;
  
  FStartTick := 0;
  FCurTick := 0;
  FTickNow := 0;
  FFrames := 0;
  FFPS := 0;

  fnext_time := 0;
  ffps := 0;
  f__fps := 0;
end;

function TelSurface.Initialize(Width, Height, ColorBits: Integer; Fullscreen: Boolean; InitFlag: TelInitFlag = ifStandard): Boolean;
var VideoInfo: PSDL_VideoInfo;
begin
  Result := true;

  FWidth := Width;
  FHeight := Height;
  FFullscreen := Fullscreen;

  {$IFDEF USE_SDL_MIXER}
  if SDL_Init(SDL_INIT_VIDEO or SDL_INIT_TIMER or SDL_INIT_AUDIO) <> 0 then
  {$ELSE}
  if SDL_Init(SDL_INIT_VIDEO or SDL_INIT_TIMER) <> 0 then
  {$ENDIF}
  begin
    WriteLn('Could not load SDL.');
    Result := false;
    Finalize;
  end;
  
  {$IFDEF USE_SDL_MIXER}
  Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 4096);
  {$ENDIF}
  
  {$IFDEF USE_SDL_TTF}
  if TTF_Init <> 0 then
  begin
  
  end;
  {$ENDIF}

  FStartTick := SDL_GetTicks;
  
  VideoInfo := SDL_GetVideoInfo;

  if VideoInfo = nil then 
  begin
    WriteLn('Could not initialize SDL.');
    Finalize;
  end;
  
  case InitFlag of
    ifStandard:
    begin
      FFlags := SDL_DOUBLEBUF or SDL_HWPALETTE;
  
      if VideoInfo.hw_available <> 0  then FFlags := FFlags or SDL_HWSURFACE
                                      else FFlags := FFlags or SDL_SWSURFACE;
    end;
    ifSoftware: FFlags := SDL_SWSURFACE;
	ifDoubleBuf: FFlags := SDL_DOUBLEBUF;
    ifHardware: FFlags := SDL_HWSURFACE;
  end;

  SDL_WM_SetCaption(PChar('ElysionLegacy application'), nil);

  SetVideoMode(Width, Height, ColorBits, Fullscreen);

  if not Assigned(Self.SDL_Surface) then
  begin
    Result := false;
    Finalize;
  end;

  fCursor := TelCursor.Create(SDL_Surface);
end;

procedure TelSurface.SetVideoMode(Width, Height, ColorBits: Integer; Fullscreen: Boolean = False);
begin
  if Width <> FWidth then FWidth := Width;
  if Height <> FHeight then FHeight := Height;
  if ColorBits <> FColorBits then FColorBits := ColorBits;

  if Fullscreen then
  begin
    FGFlags := FFlags or FFullscreenFlag;
    Self.SDL_Surface := SDL_SetVideoMode(Width, Height, ColorBits, FGFlags);
  end else Self.SDL_Surface := SDL_SetVideoMode(Width, Height, ColorBits, FFlags);
end;

procedure TelSurface.Finalize;
begin
  if FFullscreen then SDL_WM_ToggleFullScreen(Self.SDL_Surface);
  if UseBackgroundImage then SDL_FreeSurface(fBackgroundImage);
  Cursor.Free;
  SDL_FreeSurface(Self.SDL_Surface);
  
  {$IFDEF USE_SDL_MIXER}
  Mix_CloseAudio;
  {$ENDIF}
  
  SDL_Quit;
  Halt(0);
end;

procedure TelSurface.BeginScene;
begin
  if UseBackgroundImage then
  begin
    if ((fBackgroundImage.w <= fWidth) or (fBackgroundImage.h <= fHeight)) then
	  SDL_BlitSurface(fBackgroundImage, PSDLRect(0, 0, fBackgroundImage.w, fBackgroundImage.h), 
	    SDL_Surface, PSDLRect(0, 0, fBackgroundImage.w, fBackgroundImage.h))
	else
	  SDL_BlitSurface(fBackgroundImage, PSDLRect(0, 0, fWidth, fHeight), 
	    SDL_Surface, PSDLRect(0, 0, fWidth, fHeight));
  end else
    SDL_FillRect(Self.SDL_Surface, Nil, convCol(Self, FBackgroundColor));
end;

procedure TelSurface.EndScene;
begin
  DrawCursor;

  SDL_Flip(Self.SDL_Surface);
  
  HandleEvents;
end;

procedure TelSurface.EndScene(UpdateRects: array of TelRect);
//var i: Integer;
begin
  DrawCursor;

  SDL_UpdateRects(Self.SDL_Surface, High(UpdateRects), PSDLRect(UpdateRects[0].X, UpdateRects[0].Y, UpdateRects[0].W, UpdateRects[0].H));
  
  //for i := 0 to High(UpdateRects) do
  //  SDL_UpdateRect(Self.SDL_Surface, UpdateRects[i].X, UpdateRects[i].Y, UpdateRects[i].W, UpdateRects[i].H);
  
  HandleEvents;
end;

procedure TelSurface.SetFPS(FPS: Integer);
begin

end;

function TelSurface.GetFPS: Integer;
begin
  f__fps := f__fps + 1;
  fnow := SDL_GetTicks;
  if fnext_time <= fnow then
  begin
    fnext_time := fnow + 1000;
    ffps := f__fps;
    f__fps := 0;
  end;

  Result := ffps;
end;

procedure TelSurface.Quit;
begin
  FRun := false;
end;

procedure TelSurface.SetBackgroundImage(const Filename: String);
var Directory: String;
begin
  UseBackgroundImage := true;
  
  if Filename <> '' then
  begin
    Directory := ExtractFilePath(ParamStr(0));

    if FileExists(Directory+FileName) then
    begin
      {$IFDEF USE_SDL_IMAGE}
      fBackgroundImage := IMG_Load(PChar(Directory+Filename));
      {$ELSE}
      fBackgroundImage := SDL_LoadBMP(PChar(Directory+Filename));
      {$ENDIF}
	end;
  end;
end;

procedure TelSurface.SetBackgroundColor(const Color: TelColor);
begin
  UseBackgroundImage := false;
  fBackgroundColor := Color;
end;

procedure TelSurface.SetCaption(Caption: String);
begin
  FCaption := PChar(Caption);
  SDL_WM_SetCaption(FCaption, FIcon);
end;

procedure TelSurface.SetIcon(Icon: String);
var TMP_Icon: PSDL_Surface;
    Directory: String;
begin
  if Icon <> '' then
  begin
    Directory := ExtractFilePath(ParamStr(0));

    if FileExists(Icon) then
    begin
	  {$IFDEF USE_SDL_IMAGE}
      TMP_Icon := IMG_Load(PChar(Directory+Icon));
	  {$ELSE}
	  TMP_Icon := SDL_LoadBMP(PChar(Directory+Icon));
	  {$ENDIF}
	  
      FIcon := PChar(Icon);
      SDL_WM_SetIcon(TMP_Icon, 0);
    end;

  end;
  
  if TMP_Icon <> nil then SDL_FreeSurface(TMP_Icon);
end;

procedure TelSurface.HideCursor;
begin
  fHideCursor := true;
end;

procedure TelSurface.ShowCursor;
begin
  fHideCursor := false;
end;

procedure TelSurface.TakeScreenshot(const Filename: String);
begin
  SDL_SaveBMP(Self.SDL_Surface, PChar(Filename));
end;

procedure TelSurface.SetLogger(const Filename: String);
begin
  fUseLog := true;
end;

function TelSurface.GetLoggerStatus: Boolean;
begin
  Result := fUseLog;
end;

function TelSurface.OnPoint(Rect: TelRect): Boolean;
begin
  Result := OnPoint(Cursor, Rect);
end;

function TelSurface.Run: Boolean;
begin
  if FRun then Result := true
          else Result := false;
end;

procedure TelSurface.SetDelayTime;
begin

end;

procedure TelSurface.DrawCursor;
begin
  if not fHideCursor then
  begin
    if Cursor.GetSDLSurface = nil then 
	begin
      SDL_ShowCursor(1);
	  Cursor.Draw;
	end else SDL_ShowCursor(0);
  end else SDL_ShowCursor(0);
end;

procedure TelSurface.HandleEvents;
var i: Integer;
begin
   for i := 0 to High(KeyUp) do
  begin
    KeyUp[i] := false;
  end;

  for i := 0 to High(MouseButtonUp) do
  begin
    MouseButtonUp[i] := false;
  end;

  while (SDL_PollEvent(@FEvent) = 1) do
  begin

    case FEvent.type_ of
    SDL_QUITEV:
       begin
         FRun := false;
       end;

    SDL_KEYUP:
        begin
          KeyDown[FEvent.key.keysym.sym] := false;
          KeyUp[FEvent.key.keysym.sym] := true;
        End;

    SDL_KEYDOWN:
        begin
          KeyDown[FEvent.key.keysym.sym] := true;
          KeyUp[FEvent.key.keysym.sym] := false;
        end;

    SDL_MOUSEMOTION:
        begin
          SDL_GetMouseState(Cursor.Position.X, Cursor.Position.Y);
        end;
 
    SDL_MOUSEBUTTONDOWN:
        begin
          MouseButtonDown[FEvent.button.button] := true;
          MouseButtonUp[FEvent.button.button] := false;
        end;

    SDL_MOUSEBUTTONUP:
        begin
          MouseButtonDown[FEvent.button.button] := false;
          MouseButtonUp[FEvent.button.button] := true;
        end;

    end;


  end;
end;

// 
// TelObject
//

constructor TelObject.Create;
begin
  fName := '';
end;

procedure TelObject.SetName(Name: String);
begin
  fName := String;
end;

function TelObject.GetName: String;
begin
  Result := fName;
end;

//
// TelGraphicObject
//

constructor TelGraphicObject.Create;
begin
  parentSurface := nil;
end;
	
constructor Create(Surface: TelSurface);
begin
  inherited Create;

  Assign(Surface):
end;

procedure Free;
begin
  SDL_FreeSurface(parentSurface);
end;
	
procedure Assign(Surface: TelSurface);
begin
  parentSurface := Surface.SDL_Surface;
end;

function GetParentSurface: PSDL_Surface;
begin
  Result := parentSurface;
end;

//
// TelGraphicObjectExt
//

constructor TelGraphicObjectExt.Create;
begin
  inherited Create;

  fRect := makeRect(0, 0, 0, 0);
  
  fUseTransparency := false;
  fTransparencyColor := makeCol(0, 0, 0);
  
  fUseColor := false;
  fColor := makeCol(0, 0, 0);
  
  fAlpha := 255;
  fVisible := true;
end;

procedure TelGraphicObjectExt.Free;
begin
  if fSurface <> nil then SDL_FreeSurface(fSurface);
end;

function TelGraphicObjectExt.GetSDLSurface: PSDL_Surface;
begin
  Result := fSurface;
end;	
	
procedure TelGraphicObjectExt.Assign(Surface: TelSurface);
begin
  parentSurface := Surface.SDL_Surface;
end;
	
procedure TelGraphicObjectExt.SetAlpha(Alpha: Byte);
begin
  fAlpha := Alpha;
end;

function TelGraphicObjectExt.GetAlpha: Byte;
begin
  Result := fAlpha;
end;
	
procedure TelGraphicObjectExt.SetTransparency(Color: TelColor);
begin
  if fSurface <> Nil then
  begin
    fUseTransparency := true;
	fColor := Color;
	
    SDL_SetColorKey(fSurface, SDL_SRCCOLORKEY or SDL_RLEACCEL or SDL_HWACCEL,
      SDL_MapRGB(fSurface.Format, Color.R, Color.G, Color.B));  
  end;
end;

procedure TelGraphicObjectExt.SetTransparency(Coord: TelPoint);
begin
  if fSurface <> Nil then
  begin
    fUseTransparency := true;
	fColor 
  
    SDL_SetColorKey(fSurface, SDL_SRCCOLORKEY or SDL_RLEACCEL or SDL_HWACCEL,
      SDL_GetPixel(fSurface, Coord.X, Coord.Y));
  end;
end;
	
procedure TelGraphicObjectExt.SetPosition(Coord: TelPoint);
begin
  fRect.X := Coord.X;
  fRect.Y := Coord.Y;
end;
	
function TelGraphicObjectExt.GetPosition: TelPoint;
begin
  Result := makeP2D(fRect.X, fRect.Y);
end;

procedure TelGraphicObjectExt.SetWidth(Width: Integer);
begin
  fRect.W := Width;
end;

function TelGraphicObjectExt.GetWidth: Integer;
begin
  Result := fRect.W;
end;

procedure TelGraphicObjectExt.SetHeight(Height: Integer);
begin
  fRect.H := Height;
end;

function TelGraphicObjectExt.GetHeight: Integer;
begin
  Result := fRect.H;
end;
	
procedure TelGraphicObjectExt.SetRect(Rect: TelRect);
begin
  fRect := Rect;
end;

function TelGraphicObjectExt.GetRect: TelRect;
begin
  Result := fRect;
end;

procedure TelGraphicObjectExt.Move(Coord: TelPoint);
begin
  fRect.X := fRect.X + Coord.X;
  fRect.Y := fRect.Y + Coord.Y;
end;
	
procedure TelGraphicObjectExt.SetVisible(Visible: Boolean);
begin
  fVisible := Visible;
end;

function TelGraphicObjectExt.GetVisible: Boolean;
begin
  Result := fVisible;
end;
	
function TelGraphicObjectExt.OnPoint(Coord: TelPoint): Boolean;
begin
  Result := OnPoint(Coord, fRect);
end;

procedure TelGraphicObject.ColorSurface;
var i,j: Integer;
begin
  if fUseTransparency then
  begin
    for i := 0 to Self.GetWidth do
	begin
	  for j := 0 to Self.GetHeight do
	  begin
	  
	  end;
	end;
  end else 
  begin
  
  end;
end;

//
// TelGameStates
//
constructor TelGameStates.Create;
begin
  FStates := TList.Create;
end;

procedure TelGameStates.Add(Name: string);
begin
  
end;

procedure TelGameStates.Del(Name: string);
begin
  
end;

procedure TelGameStates.Del(Index: Integer);
begin
  FStates.Delete(Index);
end;

procedure TelGameStates.SetGameState(Name: string);
begin

end;

procedure TelGameStates.SetGameState(Index: Integer);
begin

end;

function TelGameStates.GetGameState(Name: string): Boolean;
begin

end;

function TelGameStates.GetGameState(Index: Integer): Boolean; 
begin

end;

//
// TelTimer
//
constructor TelTimer.Create;
begin
  FInterval := 0;
end;

constructor TelTimer.Create(Interval: Integer);
begin
  SetInterval(Interval);
end;

procedure TelTimer.SetActive(Active: Boolean);
begin
  FActive := Active;
  StartTick := SDL_GetTicks;
end;

function TelTimer.GetActive: Boolean;
begin
  Result := FActive;
end;

procedure TelTimer.SetInterval(Interval: Integer);
begin
  FInterval := Interval;
end;

function TelTimer.GetInterval: Integer;
begin
	Result := FInterval;
end;

//
// Action
// To be used like an On-/Off-Switch
function TelTimer.Action: Boolean;
begin
  Result := false;

  if Self.GetActive then
  begin
	  if FInterval <> 0 then
	  begin
      if (((SDL_GetTicks - StartTick) div FInterval) mod 2) = 0 then Result := false
                                                                else Result := true;
	  end;
  end;
end;

//
// SingleAction
// A bit shaky
// Supposed to SingleAction = true every FInterval ms
// TODO: Idea: Using tolerance < SingleAction > tolerance
function TelTimer.SingleAction: Boolean;
var diff: Integer;
begin
  Result := false;

  if Self.GetActive then
  begin
    if FInterval <> 0 then
    begin
      diff := SDL_GetTicks - StartTick;

      if ((diff mod FInterval) = 0) then Result := true
                                    else Result := false;
    end;
  end;
end;

//
// TelFont
//

procedure TelFont.Free;
begin
  inherited Free;
  
  if
end;

function TelFont.LoadFromFile(Filename: String): Boolean;
var Directory: String;
begin
  if UseAutoAssign then parentSurface := UeberSurface;
  
  Result := false;

  Directory := ExtractFilePath(ParamStr(0));
  
    if FileExists(Directory + Filename) then
    begin
      {$IFDEF USE_SDL_IMAGE}
      FFontInfo.Surface := IMG_Load(PChar(Directory + Filename));
      {$ELSE}
      FFontInfo.Surface := SDL_LoadBMP(PChar(Directory + Filename));
      {$ENDIF}
      SFont_InitFont2(@FFontInfo);
      
      Result := true;
    end else WriteLn('File does not exist: '+Filename);
    
    if FFontInfo.Surface = nil then 
    begin
      WriteLn('Could not load file: '+Filename);
      WriteLn('Error: '+{$IFDEF USE_SDL_IMAGE}IMG_GetError{$ELSE}SDL_GetError{$ENDIF});
    end;
end;

procedure TelFont.SetTransparency(Color: TelColor);
begin
  if FFontInfo.Surface <> nil then
    SDL_SetColorKey(FFontInfo.Surface, SDL_SRCCOLORKEY or SDL_RLEACCEL or SDL_HWACCEL,
      SDL_MapRGB(FFontInfo.Surface.Format, Color.R, Color.G, Color.B));
end;

procedure TelFont.SetTransparency(Coord: TelPoint);
begin
  if FFontInfo.Surface <> nil then
    SDL_SetColorKey(FFontInfo.Surface, SDL_SRCCOLORKEY or SDL_RLEACCEL or SDL_HWACCEL,
      SDL_GetPixel(FFontInfo.Surface, Coord.X, Coord.Y));
end;
    
procedure TelFont.SetAlpha(Alpha: Byte);
begin
  FAlpha := Alpha;
end;

function TelFont.GetAlpha: Byte;
begin
  Result := FAlpha;
end;
    
function TelFont.GetWidth(Text: String): Integer;
begin
  Result := SFont_TextWidth2(@FFontInfo, PChar(Text));
end;
    
function TelFont.GetHeight: Integer;
begin
  Result := FFontInfo.Surface.h;
end;

procedure TelFont.TextOut(Text: String; TextAlignment: TelAlignment = taNone);
begin
  if FAlpha <> 255 then SDL_SetAlpha(FFontInfo.Surface, SDL_SRCALPHA, fAlpha)
				   else SDL_SetAlpha(FFontInfo.Surface, SDL_SRCALPHA, fAlpha);

  case TextAlignment of
    taNone: SFont_Write2(parentSurface, @FFontInfo, Coord.X, Coord.Y, PChar(Text));
    taLeft: SFont_Write2(parentSurface, @FFontInfo, 0, Coord.Y, PChar(Text));
    taCenter: SFont_Write2(parentSurface, @FFontInfo, ((parentSurface.w - GetWidth(Text)) div 2), Coord.Y, PChar(Text));
    taRight: SFont_Write2(parentSurface, @FFontInfo, (parentSurface.w - GetWidth(Text)), Coord.Y, PChar(Text));
  end;
end;


constructor TelPrimitive.Create;
begin
  inherited Create;
  
  if UseAutoAssign then parentSurface := UeberSurface;
end;

constructor TelPrimitive.Create(Surface: TelSurface);
begin
  inherited Create;

  Assign(Surface);
end;

procedure TelPrimitive.Assign(Surface: TelSurface);
begin
  parentSurface := Surface.SDL_Surface;
end;

procedure TelPrimitive.DrawRect(Rect: TelRect; Color: TelColor);
begin
  SDL_FillRect(parentSurface, PSDLRect(Rect.X, Rect.Y, Rect.W, Rect.H), convCol(parentSurface, Color));
end;

procedure TelPrimitive.DrawLine(Origin: TelPoint; Destination: TelPoint; Color: TelColor);
begin
  SDL_DrawLine(parentSurface, Origin.X, Origin.Y, Destination.X, Destination.Y, convCol(parentSurface, Color));
end;

//
// TelSprite
//

constructor TelLayer.Create;
begin
   FCutRect := makeRect(0, 0, 0, 0);
   FRect := makeRect(0, 0, 0, 0);

   FAlpha := 255;
   FFrame := 0;
   FVisible := true;
end;

constructor TelLayer.Create(Surface: TelSurface);
begin
  Create;

  Assign(Surface);
end;

procedure TelLayer.Assign(Surface: TelSurface);
begin
  parentSurface := Surface.SDL_Surface;
end;

function TelLayer.LoadFromFile(Filename: String): Boolean;
begin
  Result := LoadFromFile(Filename, makeRect(0, 0, 0, 0));
end;

function TelLayer.LoadFromFile(Filename: String; CutRect: TelRect): Boolean;
var Directory: String;
begin
  if UseAutoAssign then parentSurface := UeberSurface;

  Result := false;

  FAnim.X := 0;
  FAnim.Y := 0;

  FCutRect := CutRect;

  if Filename <> '' then
  begin
    Directory := ExtractFilePath(ParamStr(0));

    if FileExists(Directory+FileName) then
    begin
      {$IFDEF USE_SDL_IMAGE}
      FTexture := IMG_Load(PChar(Directory+Filename));
      {$ELSE}
      FTexture := SDL_LoadBMP(PChar(Directory+Filename));
      {$ENDIF}

      if FCutRect.W = 0 then FCutRect.W := FTexture.w;
      if FCutRect.H = 0 then FCutRect.H := FTexture.h;

      CutImage(FCutRect);

      FAnim.W := GetSurfaceWidth div FCutRect.W;
      FAnim.H := GetSurfaceHeight div FCutRect.H;

      
      FMaxFrames := FAnim.W * FAnim.H;
      
	  if fSurface = nil then 
  begin
    WriteLn('Could not load image: '+Filename);
    WriteLn('Error: '+{$IFDEF USE_SDL_IMAGE}IMG_GetError{$ELSE}SDL_GetError{$ENDIF});
  end;
	  
      Result := true;
      
    end else WriteLn('File does not exist: '+Filename);
  end;
  
  
end;

function TelLayer.GetSDLSurface: PSDL_Surface;
begin
  Result := fSurface;
end;

procedure TelLayer.CutImage(Rect: TelRect);
begin
  FCutRect := Rect;
  FRect.W := FCutRect.W;
  FRect.H := FCutRect.H;
end;

function TelLayer.GetCutRect: TelRect;
begin
  Result := FCutRect;
end;

procedure TelLayer.SetPosition(Coord: TelPoint);
begin
  FRect.X := Coord.X;
  FRect.Y := Coord.Y;
end;

procedure TelLayer.Move(Coord: TelPoint);
begin
  FRect.X := FRect.X + Coord.X;
  FRect.Y := FRect.Y + Coord.Y;
end;

function TelLayer.GetPosition: TelPoint;
begin
  Result.X := FRect.X;
  Result.Y := FRect.Y;
end;

procedure TelLayer.SetZ(Z: Integer);
begin
  FZ := Z;
end;

function TelLayer.GetZ;
begin
  Result := FZ;
end;

procedure TelLayer.SetVisible(Visible: Boolean);
begin
  FVisible := Visible;
end;

function TelLayer.GetVisible: Boolean;
begin
  Result := FVisible;
end;

procedure TelLayer.SetWidth(Width: Integer);
begin
  FRect.W := Width;
end;

function TelLayer.GetWidth: Integer;
begin
  Result := FRect.W;
end;

procedure TelLayer.SetHeight(Height: Integer);
begin
  FRect.H := Height;
end;

function TelLayer.GetHeight: Integer;
begin
  Result := FRect.H;
end;

function TelLayer.GetSurfaceWidth: Integer;
begin
  Result := fSurface.w;
end;

function TelLayer.GetSurfaceHeight: Integer;
begin
  Result := fSurface.h;
end;

procedure TelLayer.SetFrame(Frame: Integer);
begin
   if Frame <= FMaxFrames then
   begin
     FFrame := Frame - 1;
     if FFrame < 0 then FFrame := 0;


     if FAnim.H = 1 then FAnim.X := FFrame
     else
     begin
       FAnim.Y := FAnim.H - 1;
       FAnim.X := FFrame - FAnim.H;
     end;

     CutImage(makeRect(FAnim.X * FCutRect.W, FAnim.Y * FCutRect.H, FCutRect.W, FCutRect.H));
   end;
end;

function TelLayer.GetFrame: Integer;
begin
  Result := FFrame;
end;

function TelLayer.GetMaxFrames: Integer;
begin
  Result := FMaxFrames;
end;

procedure TelLayer.SetAlpha(Alpha: Byte);
begin
  FAlpha := Alpha;
end;

function TelLayer.GetAlpha: Byte;
begin
  Result := FAlpha;
end;

procedure TelLayer.Draw;
begin
  if FVisible then
  begin
    if FAlpha <> 255 then SDL_SetAlpha(FTexture, SDL_SRCALPHA, FAlpha)
                     else SDL_SetAlpha(FTexture, SDL_SRCALPHA, 255);

	{$IFDEF USE_SDL_IMAGE}
	FTexture := SDL_DisplayFormatAlpha(FTexture);
	{$ELSE}
	FTexture := SDL_DisplayFormat(FTexture);
	{$ENDIF}
	

    //if (FCutRect.W <> FRect.W) or (FCutRect.H <> FRect.H) then
    //  SDL_ZoomSurface(FTexture, PSDLRect(FCutRect.X, FCutRect.Y, FCutRect.W, FCutRect.H),
    //                          parentSurface, PSDLRect(FRect.X, FRect.Y, FRect.W, FRect.H))
    //else  
      SDL_BlitSurface(FTexture, PSDLRect(FCutRect.X, FCutRect.Y, FCutRect.W, FCutRect.H),
                              parentSurface, PSDLRect(FRect.X, FRect.Y, FCutRect.W, FCutRect.H));
                              
  end;

end;

procedure TelLayer.Animate(Timer: TelTimer; Loop: Boolean = True);
var tmp_frame: Integer;
begin
  if Timer.SingleAction then FFrame := FFrame + 1;
  if Loop then if GetFrame = GetMaxFrames then FFrame := 0;

  Self.SetFrame(FFrame);
end;

function TelLayer.OnPoint(Coord: TelPoint): Boolean;
begin
  if (Coord.X >= FRect.X) and
     (Coord.Y >= FRect.Y) and
     (Coord.X < (FRect.X + FRect.W)) and
     (Coord.Y < (FRect.Y + FRect.H)) then Result := true else Result := false;
end;

procedure TelLayer.Free;
begin
  if FTexture <> nil then SDL_FreeSurface(FTexture);
end;

{$IFDEF USE_SDL_MIXER}

//
// TelMusic
//

constructor TelMusic.Create;
begin
  inherited Create;
end;

constructor TelMusic.Create(Filename: String);
begin
  inherited Create;
  LoadFromFile(Filename);
end;

procedure TelMusic.LoadFromFile(Filename: String);
begin

end;

//
// TelSound
//

constructor TelSound.Create;
begin
  inherited Create;
end;

constructor TelSound.Create(Filename: String);
begin
  inherited Create;
  LoadFromFile(Filename);
end;

Procedure TelSound.LoadAudioFile(Filename: String);
Begin
  fSound := Mix_LoadWAV(PChar(Filename));
End;

Procedure TelSound.Free;
Begin
  inherited Free;

  Mix_FreeChunk(fSound);
  Mix_CloseAudio;
End;

Procedure TelSound.Play;
Begin
  Mix_PlayChannel(0, FSound, 0);
End;

Procedure TelSound.Stop;
Begin
  Mix_HaltChannel(0);
End;


{$ENDIF}

end.
