(* Stickman's Escape

   Ludum Dare #14 Advancing Wall of Doom

   Copyright (c) 2009 Freeze.Dev (Johannes Stein)

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation
  files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use,
  copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  OTHER DEALINGS IN THE SOFTWARE.

  info@freeze-dev.de
  www.freeze-dev.de

 V1.0  (09-04-19)  Initial release

 *)

program escape;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

uses ElysionLegacy,
     SysUtils;

type
  TGameStates = (gsMainMenu, gsGame);

const
  WIDTH = 800;
  HEIGHT = 600;
  BPP = 32;
  
  MinLevel = 1;
  MaxLevel = 3;

var Surface: TelSurface;
	Fullscreen: Boolean = false;
	
	GameState: TGameStates = gsMainMenu;
	
    Background, BackgroundCave, Block, GUI, GUI_Gem, Door, Player, Cursor: TelSprite;
	Gem: array[1..5] of TelSprite;
	WarpPoints: array[1..5] of TelSprite;
	Platforms: array[1..10] of TelSprite;
	
	Font, SmallFont: TelFont;
	Timer: TelTimer;
	
	Info: Boolean = false;
	GameOver: Boolean = false;
	StopMotion: Boolean = false;
	
	TutorialTips: Boolean = false;
	Congratz: Boolean = false;
	
	Points: Integer;
	GemsCollected: Byte;
	
	CurLevel: Byte = 1;
	WinLevel: array[MinLevel..MaxLevel] of Boolean;
	DrawThem: Integer;
	

procedure CreateObjects;
var i: Integer;
begin
  Surface := TelSurface.Create;
  Background := TelSprite.Create;
  BackgroundCave := TelSprite.Create;
  Block := TelSprite.Create;
  GUI := TelSprite.Create;
  GUI_Gem := TelSprite.Create;
  Door := TelSprite.Create;
  Player := TelSprite.Create;
  for i := 1 to 5 do 
  begin
    Gem[i] := TelSprite.Create;
	WarpPoints[i] := TelSprite.Create;
	Platforms[i] := TelSprite.Create;
	Platforms[i + 5] := TelSprite.Create;
  end;
  Cursor := TelSprite.Create;
  
  Font := TelFont.Create;
  SmallFont := TelFont.Create;
  
  Timer := TelTimer.Create;
end;

procedure AssignObjects;
var i: Integer;
begin
  // Assign our image to the surface
  Background.Assign(Surface);
  BackgroundCave.Assign(Surface);
  Block.Assign(Surface);
  GUI.Assign(Surface);
  GUI_Gem.Assign(Surface);
  Door.Assign(Surface);
  Player.Assign(Surface);
  for i := 1 to 5 do 
  begin
    Gem[i].Assign(Surface);
	WarpPoints[i].Assign(Surface);
	Platforms[i].Assign(Surface);
	Platforms[i + 5].Assign(Surface);
  end;
  
  Cursor.Assign(Surface);
  
  Font.Assign(Surface);
  SmallFont.Assign(Surface);
end;

procedure LoadImages;
var i: Integer;
begin
  // Load the image
  Background.LoadFromFile('../media/background.jpg');
  BackgroundCave.LoadFromFile('../media/cave.png');
  Block.LoadFromFile('../media/block.jpg');
  GUI.LoadFromFile('../media/gui.bmp');
  GUI_Gem.LoadFromFile('../media/gem.bmp');
  Door.LoadFromFile('../media/door.bmp', makeRect(0, 0, 40, 40));
  Player.LoadFromFile('../media/human.bmp', makeRect(0, 0, 40, 40));
  for i := 1 to 5 do 
  begin
    Gem[i].LoadFromFile('../media/gem.bmp');
	WarpPoints[i].LoadFromFile('../media/warpspace.bmp');
	Platforms[i].LoadFromFile('../media/platform.bmp');
	Platforms[i + 5].LoadFromFile('../media/platform.bmp');
  end;

  Cursor.LoadFromFile('../media/cursor.bmp');
  
  Font.LoadFromFile('../media/StoneFont.png');
  SmallFont.LoadFromFile('../media/SmallStone.png');

  GUI.SetTransparency(makeP2D(0, 0));
  GUI_Gem.SetTransparency(makeP2D(0, 0));
  Player.SetTransparency(makeP2D(0, 0));
  Door.SetTransparency(makeP2D(0, 0));
  
  for i := 1 to 5 do 
  begin
    Gem[i].SetTransparency(makeP2D(0, 0));
	WarpPoints[i].SetTransparency(makeP2D(0, 0));
	Platforms[i].SetTransparency(makeP2D(0, 0));
	Platforms[i + 5].SetTransparency(makeP2D(0, 0));
  end;
  
  Cursor.SetTransparency(makeCol(255, 255, 255));
  
  Background.SetPosition(makeP2D(0, 0));
  BackgroundCave.SetPosition(makeP2D(0, 0));
  
  GUI.SetPosition(makeP2D(WIDTH - 170, 0));
  GUI.SetAlpha(128);
  GUI_Gem.SetPosition(makeP2D(WIDTH - 155, 25));
  
  Background.Generate;
  BackgroundCave.Generate;
  Block.Generate;
  GUI.Generate;
  GUI_Gem.Generate;
  Door.Generate;
  Player.Generate;
  for i := 1 to 5 do 
  begin
    Gem[i].Generate;
	WarpPoints[i].Generate;
	Platforms[i].Generate;
	Platforms[i + 5].Generate;
  end;
  
  Cursor.Generate;
end;

procedure FreeObjects;
var i: Integer;
begin
  Background.Free;
  BackgroundCave.Free;
  Block.Free;
  GUI.Free;
  GUI_Gem.Free;
  Door.Free;
  Player.Free;
  for i := 1 to 5 do 
  begin
    Gem[i].Free;
	WarpPoints[i].Free;
	Platforms[i].Free;
	Platforms[i + 5].Free;
  end;
  
  Cursor.Free;
  Surface.Finalize;
end;

function CheckCollision(Rect1, Rect2: TelSprite): Boolean;
begin
    if ((Rect1.GetRect.X = Rect2.GetRect.X)  and
	   (Rect1.GetRect.Y = Rect2.GetRect.Y)) then Result := true else Result := false;
end;

function CheckPlatforms: Boolean;
var i: Integer;
begin
  Result := false;
  for i := 1 to 10 do 
    if Platforms[i].GetVisible then 
	  if Player.GetPosition.Y = Platforms[i].GetPosition.Y then Result := true; 
end;

function MoveRightAllowed: Boolean;
var i, tmp: Integer;
begin
  Result := false;
  for i := 1 to 10 do 
    if Platforms[i].GetVisible then 
	  if Player.GetPosition.Y = Platforms[i].GetPosition.Y then tmp := i;
  
  if CheckCollision(Player, Platforms[tmp]) then Result := false
										    else Result := true;
end;

function MoveLeftAllowed: Boolean;
var i, tmp: Integer;
begin
  Result := false;
  for i := 10 downto 1 do 
    if Platforms[i].GetVisible then 
	  if Player.GetPosition.Y = Platforms[i].GetPosition.Y then tmp := i;
  
  if CheckCollision(Player, Platforms[tmp]) then Result := false
										    else Result := true;
end;

procedure ResetGame;
var i: Integer;
begin
  // Reset all variables
  Points := 0;
  GemsCollected := 0;
  CurLevel := 1;
  
  for i := MinLevel to MaxLevel do WinLevel[i] := false;
end;

procedure ResetLevel;
var i: Integer;
begin
  Player.SetPosition(makeP2D(0, 600));
  Door.SetPosition(makeP2D(0, 640)); 
  for i := 1 to 5 do
  begin
    Gem[i].SetVisible(false);
	Gem[i].SetPosition(makeP2D(0, 0));
	
	WarpPoints[i].SetVisible(false);
	WarpPoints[i].SetPosition(makeP2D(0, 0));
	WarpPoints[i].SetName('');
	
    Platforms[i].SetVisible(false);
	Platforms[i].SetPosition(makeP2D(0, 0));
	
	Platforms[i + 5].SetVisible(false);
	Platforms[i + 5].SetPosition(makeP2D(0, 0));
  end;

  // Ran into some problem loading the levels from text files
  // so now it's hardcoded
  if CurLevel = 1 then
  begin
    Player.SetPosition(makeP2D(80, 560));
    Door.SetPosition(makeP2D(640, 560)); 
  
    Gem[1].SetVisible(true);
	Gem[1].SetPosition(makeP2D(320, 560));
  end;
  if CurLevel = 2 then
  begin
    Player.SetPosition(makeP2D(160, 280));
    Door.SetPosition(makeP2D(480, 560)); 
    
	Platforms[1].SetVisible(true);
	Platforms[2].SetVisible(true);
	Platforms[3].SetVisible(true);
	Platforms[4].SetVisible(true);
	
	Platforms[1].SetPosition(makeP2D(160, 280));
	Platforms[2].SetPosition(makeP2D(200, 280));
	Platforms[3].SetPosition(makeP2D(400, 400));
	Platforms[4].SetPosition(makeP2D(440, 400));
	
	WarpPoints[1].SetVisible(true);
	WarpPoints[2].SetVisible(false);
	WarpPoints[1].SetName('Trigger');
	
	WarpPoints[1].SetPosition(makeP2D(400, 400));
	WarpPoints[2].SetPosition(makeP2D(160, 560));
	  
    Gem[1].SetVisible(true);
	Gem[1].SetPosition(makeP2D(440, 400));
  end;
  if CurLevel = 3 then
  begin
    Player.SetPosition(makeP2D(200, 160));
    Door.SetPosition(makeP2D(520, 560)); 
    
	Platforms[1].SetVisible(true);
	Platforms[2].SetVisible(true);
	Platforms[3].SetVisible(true);
	Platforms[4].SetVisible(true);
	Platforms[5].SetVisible(true);
	Platforms[6].SetVisible(true);
	Platforms[7].SetVisible(true);
	Platforms[8].SetVisible(true);
	
	Platforms[1].SetPosition(makeP2D(120, 160));
	Platforms[2].SetPosition(makeP2D(160, 160));
	Platforms[3].SetPosition(makeP2D(200, 160));
	Platforms[4].SetPosition(makeP2D(240, 160));
	Platforms[5].SetPosition(makeP2D(280, 160));
	Platforms[6].SetPosition(makeP2D(600, 320));
	Platforms[7].SetPosition(makeP2D(640, 320));
	Platforms[8].SetPosition(makeP2D(240, 400));
	
	WarpPoints[1].SetVisible(true);
	WarpPoints[2].SetVisible(false);
	WarpPoints[3].SetVisible(false);
	WarpPoints[1].SetName('Trigger');
	WarpPoints[2].SetName('Trigger');
	
	WarpPoints[1].SetPosition(makeP2D(600, 320));
	WarpPoints[2].SetPosition(makeP2D(240, 400));
	WarpPoints[3].SetPosition(makeP2D(440, 560));
	  
    Gem[1].SetVisible(true);
	Gem[1].SetPosition(makeP2D(120, 160));
	Gem[2].SetVisible(true);
	Gem[2].SetPosition(makeP2D(280, 160));
	Gem[3].SetVisible(true);
	Gem[3].SetPosition(makeP2D(640, 320));
  end;
 
  Block.SetPosition(makeP2D(0, -600));
  Player.SetVisible(true);
  
  Timer.SetActive(true);
  Door.CutImage(makeRect(0, 0, 40, 40));
  
  GameOver := false;
  StopMotion := false;
end;

procedure DoGameOver;
begin
  Timer.SetActive(false);
  Font.TextOut(0, ((HEIGHT - Font.GetHeight) div 2), 'Game Over', taCenter);
  SmallFont.TextOut(0, 350, 'Press SPACE to repeat this level or ESC to return back to the main menu', taCenter);
  
  GameOver := true;
  StopMotion := true;
end;

procedure DoWinLevel;
begin
  Timer.SetActive(false);
  Door.CutImage(makeRect(40, 0, 40, 40));
  Font.TextOut(0, ((HEIGHT - Font.GetHeight) div 2), 'Level finished', taCenter);
  SmallFont.TextOut(0, 350, 'Press SPACE to jump to the next level or ESC to return back to the main menu', taCenter);
  
  Player.SetVisible(false);
  WinLevel[CurLevel] := true;
  StopMotion := true;
end;

procedure DoGameLogic;
var i: Integer;
begin
  for i := 1 to 5 do begin
    if Gem[i].GetVisible then
	begin
	  if CheckCollision(Player, Gem[i]) then 
      begin
        Gem[i].SetVisible(false);
	    GemsCollected := GemsCollected + 1;
      end;
	end;
	if WarpPoints[i].GetVisible then
	begin
	  if WarpPoints[i].OnPoint(Surface.Cursor) then
	  begin
	    if Surface.MouseButtonUp[BUTTON_LEFT] then 
	    begin
	      if GemsCollected > 0 then
		  begin
	        Player.SetPosition(WarpPoints[i].GetPosition);
			WarpPoints[i].SetVisible(false);
			if WarpPoints[i].GetName = 'Trigger' then WarpPoints[i + 1].SetVisible(true);
		    GemsCollected := GemsCollected - 1;
		  end;
	    end;
	  end;
	end;
  end;
  
  if CheckCollision(Player, Door) then 
  begin
    //Player.SetPosition(makeP2D(0, 600));
    DoWinLevel;
  end else
  begin
    if (Block.GetPosition.Y + 600 >= Player.GetPosition.Y) then DoGameOver;
  end;
  
 
  
  
  if not StopMotion then
  begin 
    if CheckPlatforms then
	begin
	  if Surface.KeyDown[K_LEFT] then if MoveLeftAllowed then Player.Move(makeP2D(-1, 0));
      if Surface.KeyDown[K_RIGHT] then if MoveRightAllowed then Player.Move(makeP2D(1, 0)); 
	end;
    if (Player.GetPosition.Y = HEIGHT - 40) then
	begin
	  if Surface.KeyDown[K_LEFT] then if Player.GetPosition.X > 40 then Player.Move(makeP2D(-1, 0));
      if Surface.KeyDown[K_RIGHT] then if Player.GetPosition.X < WIDTH - 120 then Player.Move(makeP2D(1, 0));
	end;
  end;
end;

begin
  CreateObjects;
  Surface.Initialize(WIDTH, HEIGHT, BPP, Fullscreen);
  AssignObjects;
  Timer.SetInterval(3);
  LoadImages;
  Surface.SetCaption('LD48 - Stickman''s Escape');
  
  // Setting the game loop
  while Surface.Run do
  begin
    Surface.BeginScene;
      
      // Press ESC to close the application
      if Surface.KeyUp[K_ESCAPE] then 
	  begin
	    if GameState = gsMainMenu then Surface.Quit
								  else GameState := gsMainMenu;
	  end;
	  
	  if Surface.KeyUp[K_SPACE] then
	  begin
	    if GameState = gsGame then
		begin
		  if GameOver then ResetLevel;
		  if CurLevel <> MaxLevel then
		  begin
		    if WinLevel[CurLevel] then 
		    begin
		      CurLevel := CurLevel + 1;
			  Points := Points + (- Block.GetPosition.Y);
			  ResetLevel;
		    end;
		  end else
		  begin
		    GameState := gsMainMenu;
			Congratz := true;
		  end;
		end;
	  end;
	  
	  if Surface.KeyUp[K_F] then Info := not Info;
	  
	  if Surface.KeyDown[K_W] then
	  begin
	    Fullscreen := not Fullscreen;
		Surface.SetVideoMode(WIDTH, HEIGHT, BPP, Fullscreen);
	  end;
	  
	  if Timer.SingleAction then Block.Move(makeP2D(0, 1));
	  
      // Draw the image
	  Background.Draw;
	  
	  case GameState of
	    gsMainMenu:
	    begin
	      Font.TextOut(0, 280, 'New Game', taCenter);
		  Font.TextOut(0, 320, 'Quit', taCenter);
		  
		  if Congratz then 
		  begin
		    Font.TextOut(0, 420, 'Congratulations: You mastered the game.', taCenter);
			SmallFont.TextOut(0, 450, 'Stay tuned for future releases.', taCenter);
		  end;
		  
		  if OnPoint(Surface.Cursor, makeRect(((WIDTH - Font.GetWidth('New Game')) div 2), 280, Font.GetWidth('New Game'), Font.GetHeight)) then
		  begin
		    if Surface.MouseButtonDown[BUTTON_LEFT] then 
			begin
			  GameState := gsGame;
			  ResetGame;
			  ResetLevel;
			end;
		  end;
		  
		  if OnPoint(Surface.Cursor, makeRect(((WIDTH - Font.GetWidth('Quit')) div 2), 320, Font.GetWidth('Quit'), Font.GetHeight)) then
		  begin
		    if Surface.MouseButtonDown[BUTTON_LEFT] then Surface.Quit;
		  end;
	    end;
	  
	    gsGame:
	    begin
	      Door.Draw;
	      
		  for DrawThem := 1 to 5 do 
		  begin
			Gem[DrawThem].Draw;
			WarpPoints[DrawThem].Draw;
			Platforms[DrawThem].Draw;
			Platforms[DrawThem + 5].Draw;
		  end;
		  
		  Player.Draw;
		  
		  Block.Draw;
		  BackgroundCave.Draw;
		  
		  GUI.Draw;
		  SmallFont.TextOut(0, 10, 'Level: '+IntToStr(CurLevel)+'   ', taRight);
		  SmallFont.TextOut(0, 25, 'Points: '+IntToStr(Points)+'   ', taRight);
		  GUI_Gem.Draw;
		  SmallFont.TextOut(0, 40, 'Warp Gems: '+IntToStr(GemsCollected)+'   ', taRight);
		  
		  DoGameLogic;
	    end;
	  end;
	  
	  if Info then
	  begin
	    SmallFont.TextOut(0, 0, 'FPS: ' + IntToStr(Surface.GetFPS));
		SmallFont.TextOut(0, 590, '(C) Freeze.Dev (Johannes Stein) 2009 | http://www.freeze-dev.de | LD48 #14 entry');
	  end;
	  
	  Cursor.SetPosition(makeP2D(Surface.Cursor.X, Surface.Cursor.Y));
	  Cursor.Draw;
      
    Surface.EndScene;
  end;
  FreeObjects;
end.
